//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this library; see the file COPYING.LIB.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//
#include "kvi_app.h"

//============ KviAliasDialog ============//

KviAliasDialog::KviAliasDialog(KviFrame *lpFrame):QDialog(lpFrame,"KviAliasDialog",true)
{
	_debug_entertrace("KviAliasDialog");

	m_lpFrm=lpFrame;
	m_lpMng=lpFrame->m_lpAliasManager;

	setCaption("Aliases");
	setMinimumWidth(400);
	setMinimumHeight(400);


	m_lpEdit=new KviMultiLineEdit(this,lpFrame);
	m_lpDone=new QPushButton("Done",this);
	m_lpBox =new QComboBox(false,this);

	connect(m_lpDone,SIGNAL(clicked()),this,SLOT(accept()));
	connect(m_lpBox,SIGNAL(activated(const char *)),this,SLOT(selectionChanged(const char *)));

	m_lpRename=new QPushButton("Rename...",this);
	m_lpRemove=new QPushButton("Remove...",this);
	m_lpNew=new QPushButton("New...",this);

	connect(m_lpNew,SIGNAL(clicked()),this,SLOT(newAlias()));
	connect(m_lpRemove,SIGNAL(clicked()),this,SLOT(removeAlias()));
	connect(m_lpRename,SIGNAL(clicked()),this,SLOT(renameAlias()));

	m_lpLined=new KRestrictedLine(this,"Mah","qwertyuioplkjhgfdsazxcvbnm.QWERTYUIOPLKJHGFDSAZXCVBNM");
	m_lpLined->hide();

	connect(m_lpLined,SIGNAL(returnPressed()),this,SLOT(enterPressed()));

	m_iMode=_KVI_ALIDLG_MODE_NONE;
	fillBox();
	resize(400,400);
	if(!m_lpMng->m_lpAliasList->isEmpty()){
		m_szLastSelection="";
		m_lpBox->setCurrentItem(0);
		selectionChanged(m_lpMng->m_lpAliasList->first()->szName.data());
	} else m_szLastSelection="";

	_debug_leavetrace("KviAliasDialog");
}

//============ ~KviAliasDialog ============//

KviAliasDialog::~KviAliasDialog()
{
	_debug_entertrace("~KviAliasDialog");
	selectionChanged(0);
	m_lpMng->saveToFile();
	delete m_lpEdit;
	delete m_lpDone;
	delete m_lpBox;
	delete m_lpRename;
	delete m_lpRemove;
	delete m_lpNew;
	delete m_lpLined;
	_debug_leavetrace("~KviAliasDialog");
}


//============ newAlias ============//

void KviAliasDialog::newAlias()
{
	_debug_entertrace("newAlias");
	enableLined(false);
	m_iMode=_KVI_ALIDLG_MODE_NEW;
	_debug_leavetrace("newAlias");
}


//============ removeAlias ============//

void KviAliasDialog::removeAlias()
{
	_debug_entertrace("removeAlias");
	const char *szAli=m_lpBox->currentText();
	if(!szAli)return;
	KviAliasStruct *lpS=0;
	KviAliasStruct *lpF=0;
	for(lpS=m_lpMng->m_lpAliasList->first();lpS && (!lpF);lpS=m_lpMng->m_lpAliasList->next()){
		if(!strcasecmp(lpS->szName.data(),szAli))lpF=lpS;
	}
	if(lpF){
		m_lpMng->m_lpAliasList->removeRef(lpF);
		m_szLastSelection="";
		fillBox();
		if(!m_lpMng->m_lpAliasList->isEmpty()){
			m_lpBox->setCurrentItem(0);
			selectionChanged(m_lpMng->m_lpAliasList->first()->szName.data());
		}	
	}
	_debug_leavetrace("removeAlias");
}


//============ renameAlias ============//

void KviAliasDialog::renameAlias()
{
	_debug_entertrace("renameAlias");
	const char *szAli=m_lpBox->currentText();
	if(!szAli)return;
	KviAliasStruct *lpS=0;
	KviAliasStruct *lpF=0;
	for(lpS=m_lpMng->m_lpAliasList->first();lpS && (!lpF);lpS=m_lpMng->m_lpAliasList->next()){
		if(!strcasecmp(lpS->szName.data(),szAli))lpF=lpS;
	}
	if(lpF){
		m_szTmpBuf=lpF->szBuffer.copy();
		m_szTmpName=lpF->szName.copy();
		m_lpMng->m_lpAliasList->removeRef(lpF);
		m_szLastSelection="";
		enableLined(false);
		m_lpLined->setText(szAli);
		m_iMode=_KVI_ALIDLG_MODE_RENAME;	
	}

	_debug_leavetrace("renameAlias");
}


//============ fillBox ============//

void KviAliasDialog::fillBox()
{
	_debug_entertrace("fillBox");
	if(m_lpMng->m_lpAliasList->isEmpty()){
		m_lpBox->setEnabled(false);
		m_lpEdit->clear();
		m_lpEdit->setEnabled(false);
		m_lpRename->setEnabled(false);
		m_lpRemove->setEnabled(false);	
	} else {
		KviAliasStruct *lpS=0;
		m_lpBox->clear();
		for(lpS=m_lpMng->m_lpAliasList->first();lpS;lpS=m_lpMng->m_lpAliasList->next()){
			m_lpBox->insertItem(lpS->szName.data());
		}
	}
	_debug_leavetrace("fillBox");
}

//============ selectionChanged ============//

void KviAliasDialog::selectionChanged(const char *szSel)
{
	_debug_entertrace("selectionChanged");
	//debug("WE ARE HERE:%s",m_szLastSelection.data());
	KviAliasStruct *lpS=0;
	KviAliasStruct *lpF=0;
	for(lpS=m_lpMng->m_lpAliasList->first();lpS && (!lpF);lpS=m_lpMng->m_lpAliasList->next()){
		//debug("LOOKING %s",lpS->szName.data());
		if(lpS->szName==m_szLastSelection)lpF=lpS;
	}
	//debug("DONE LOOKING");
	if(lpF){
		//debug("FOUND %s",lpS->szBuffer.data());
		lpF->szBuffer=m_lpEdit->text();
		//debug("BUFFER SET %s",lpS->szBuffer.data());
		if(lpF->szBuffer.isEmpty())m_lpMng->m_lpAliasList->removeRef(lpF);
		//debug("DONE FOUND");
	}
	if(!szSel || (!(*szSel)))return;
	lpF=0;
	m_szLastSelection=szSel;
	m_szLastSelection=m_szLastSelection.upper();
	for(lpS=m_lpMng->m_lpAliasList->first();lpS && (!lpF);lpS=m_lpMng->m_lpAliasList->next()){
		if(lpS->szName==m_szLastSelection)lpF=lpS;
	}
	if(lpF){
		m_lpEdit->setText(lpF->szBuffer);
		m_lpEdit->setFocus();
	} else {
		debug("Qt drunk???????");
		ASSERT(false);
	}
	_debug_leavetrace("selectionChanged");
}



//============ enterPressed ============//

void KviAliasDialog::enterPressed()
{
	_debug_entertrace("enterPressed");
	if(m_iMode==_KVI_ALIDLG_MODE_NEW){
		if(m_lpLined->text()){
			KviAliasStruct *lpS=0;
			KviAliasStruct *lpF=0;
			for(lpS=m_lpMng->m_lpAliasList->first();lpS && (!lpF);lpS=m_lpMng->m_lpAliasList->next()){
				if(!strcasecmp(lpS->szName.data(),m_lpLined->text()))lpF=lpS;
			}
			if(lpF){
				QMessageBox::warning(this,"New alias...","This name is already in the alias list.","&OK");
				newAlias();
				return;
			}	
			lpS=new KviAliasStruct;
			lpS->szName=QString(m_lpLined->text()).upper();
			lpS->szBuffer="/<\n    echo $_window empty alias;\n>;";
			m_lpMng->m_lpAliasList->append(lpS);
			fillBox();
			m_lpBox->setCurrentItem(m_lpBox->count()-1);
			selectionChanged(lpS->szName.data());
		}
	} else if(m_iMode==_KVI_ALIDLG_MODE_RENAME){
		if(m_lpLined->text()){
			KviAliasStruct *lpS=0;
			KviAliasStruct *lpF=0;
			for(lpS=m_lpMng->m_lpAliasList->first();lpS && (!lpF);lpS=m_lpMng->m_lpAliasList->next()){
				if(!strcasecmp(lpS->szName.data(),m_lpLined->text()))lpF=lpS;
			}
			if(lpF){
				QMessageBox::warning(this,"Rename alias...","This name is already in the alias list.","&OK");
			}	
			lpS=new KviAliasStruct;
			if(!lpF)lpS->szName=QString(m_lpLined->text()).upper();
			else lpS->szName=m_szTmpName.copy();
			lpS->szBuffer=m_szTmpBuf.copy();
			m_lpMng->m_lpAliasList->append(lpS);
			fillBox();
			m_lpBox->setCurrentItem(m_lpBox->count()-1);
			selectionChanged(lpS->szName.data());
		} else {
			fillBox();
			if(!m_lpMng->m_lpAliasList->isEmpty()){
				m_lpBox->setCurrentItem(0);
				selectionChanged(m_lpMng->m_lpAliasList->first()->szName.data());
			}
			return;
		}	
	}
	enableLined(true);
	m_iMode=_KVI_ALIDLG_MODE_NONE;
	_debug_leavetrace("enterPressed");
}


//============ enable ============//

void KviAliasDialog::enableLined(bool bEnable)
{
	_debug_entertrace("enable");
	m_lpBox->setEnabled(bEnable);
	m_lpRename->setEnabled(bEnable);
	m_lpRemove->setEnabled(bEnable);
	m_lpNew->setEnabled(bEnable);
	//m_lpDone->setEnabled(bEnable);
	m_lpEdit->setEnabled(bEnable);
	if(bEnable){
		m_lpLined->hide();
		m_lpBox->show();
		m_lpEdit->setFocus();
	} else {
		m_lpBox->hide();
		m_lpLined->show();
		m_lpLined->setFocus();
	}
	_debug_leavetrace("enable");
}



//============ resizeEvent ============//

void KviAliasDialog::resizeEvent(QResizeEvent *)
{
	_debug_entertrace("resizeEvent");

	m_lpDone->setGeometry(width()-100,height()-30,95,25);
	m_lpBox->setGeometry(5,5,width()-10,25);
	m_lpRename->setGeometry(15+(((width()-20)/3)*2),40,(width()-20)/3,25);
	m_lpRemove->setGeometry(10+((width()-20)/3),40,(width()-20)/3,25);
	m_lpNew->setGeometry(5,40,(width()-20)/3,25);
	m_lpLined->setGeometry(5,5,width()-10,25);
	m_lpEdit->setGeometry(5,70,width()-10,height()-105);
	_debug_leavetrace("resizeEvent");
}



#include "m_kvi_alidlg.moc"
