//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//

//#define _KVI_DEBUG_CLASS_NAME_ "KviCtcp"

#include "kvi_app.h"

#include "kvi_token.h"
#include "kvi_ctcp.h"

#include <kfiledialog.h>


//============ KviCTCP ============//

KviCTCP::KviCTCP(KviFrame *lpFrame,KviServerParser *lpPar) : QObject(lpFrame)
{
	_debug_entertrace("KviCTCP");
	m_lpFrm       = lpFrame;
	m_lpGlb       = m_lpFrm->m_lpGlb;
	m_lpUsr       = m_lpFrm->m_lpUsr;
	m_lpOpt       = m_lpFrm->m_lpOpt;
	m_lpSock      = m_lpFrm->m_lpSock;
	m_iCTCPCount  = 0;
	m_lpPar       = lpPar;
	m_lpCTCPTimer = new QTimer(this);
	connect(m_lpCTCPTimer,SIGNAL(timeout()),this,SLOT(resetCTCPCount()));
	_debug_leavetrace("KviCTCP");
}

//============ ~KviCTCP ============//

KviCTCP::~KviCTCP()
{
	_debug_entertrace("~KviCTCP");
	if(m_lpCTCPTimer->isActive())m_lpCTCPTimer->stop();
	delete m_lpCTCPTimer;
	_debug_leavetrace("~KviCTCP");
}

//============ replyCTCPErrormsg ============//

void KviCTCP::replyCTCPErrormsg(QString &szNick,const char *szReply)
{
	_debug_entertrace("replyCTCPErrormsg");
	m_lpSock->sendFmtData("NOTICE %s :%c%s%c",szNick.data(),0x01,szReply,0x01);
	_debug_leavetrace("replyCTCPErrormsg");
}

//============ handleCTCPActionRequest ============//

void KviCTCP::handleCTCPActionRequest(QString &szNick,QString &target,KviIrcToken *lpData)
{
	_debug_entertrace("handleCTCPActionRequest");
	if((target[0]=='#')||(target[0]=='&')){
		m_lpPar->findWin(target.data())->doFmtOutput(KVI_OUT_ACTION,"%s %s",szNick.data(),lpData->szString.data());
	} else m_lpPar->topWin()->doFmtOutput(KVI_OUT_ACTION,"%s %s",szNick.data(),lpData->szString.data());
	_debug_leavetrace("handleCTCPActionRequest");
}

//============ handleCTCPPageRequest ============//

void KviCTCP::handleCTCPPageRequest(QString &szNick,QString &target,KviIrcToken *lpData)
{
	_debug_entertrace("handleCTCPPageRequest");
	if((target[0]=='#')||(target[0]=='&')){
		m_lpPar->findWin(target.data())->doFmtOutput(KVI_OUT_CTCP,"[PAGE from %s] %s",szNick.data(),lpData->szString.data());
	} else m_lpPar->topWin()->doFmtOutput(KVI_OUT_CTCP,"[PAGE from %s] %s",szNick.data(),lpData->szString.data());
	_debug_leavetrace("handleCTCPPageRequest");
}

//============ handleDCCChat ============//

void KviCTCP::handleDCCChat(QString &szNick,QString &szMask,QString &szPara,QString &szAddr,QString &szPort)
{
	_debug_entertrace("handleDCCChat");
	if(!m_lpOpt->bIgnoreDCCChat){
		QString szReq=szNick+" ["+szMask+"]\nrequests a DCC CHAT ["+szAddr+" : "+szPort+"]\nDo you accept?";
		if(!m_lpOpt->bAutoAcceptDCCChat){
			if(QMessageBox::information(m_lpFrm,"DCC CHAT Request",szReq.data(),"&Yes","&No")==0){
				QString szChatName="[DCC-CHAT]:-"+szNick;		
				KviChatWnd *lpWnd=m_lpFrm->createChatWnd(szChatName.data());
				lpWnd->acceptChat(szPort,szAddr,szNick.data());
			} else m_lpPar->m_lpCons->doFmtOutput(KVI_OUT_INTERNAL,"Refusing DCC CHAT from %s [%s]",szNick.data(),szMask.data());
		} else {
			QString szChatName="[DCC-CHAT]:-"+szNick;
			KviChatWnd *lpWnd=m_lpFrm->createChatWnd(szChatName.data());
			lpWnd->acceptChat(szPort,szAddr,szNick.data());			
		}
	} else replyCTCPErrormsg(szNick,"DCC CHAT protocol is currently disabled. (user request)");
	_debug_leavetrace("handleDCCChat");
}

//============ getDCCSaveFileName ============//

bool KviCTCP::getDCCSaveFileName(QString &szNewFileName,QString &szOldFileName,QString &szFileSize)
{
	_debug_entertrace("getDCCSaveFileName");
	QString szHome(getKVIrcHomeDirectory("Incoming"));
	QDir aDir(szHome.data());
	if(!aDir.exists())aDir.mkdir(szHome.data(),true);
	//and set it as initial directory
	KFileDialog aFileDialog(szHome.data(),0,m_lpFrm,0,true);
	aFileDialog.setCaption("Choose a file name to save...");
	aFileDialog.setSelection(szOldFileName.data());
	m_lpFrm->m_bFocusManagingEnabled=false;
	aFileDialog.exec();
	m_lpFrm->m_bFocusManagingEnabled=true;
	szNewFileName=QString (aFileDialog.selectedFile());

	bool bOK=false;
	uint uFileSize=szFileSize.toUInt(&bOK);
	if(!bOK)uFileSize=0;

	if(!szNewFileName.isEmpty()){
		QFile testFile(szNewFileName.data());
		bool bExists=testFile.exists();
		if(bExists){
			//check the file size;
			bool bShouldResume=false;
			QFileInfo testFInfo(testFile);
			QString szShortFileName=testFInfo.fileName();
			if(!strcmp(szShortFileName.data(),szOldFileName.data())){
				//selected the same file name!..if the size that whe have is smaller than the offered one...
				if(testFInfo.size()<uFileSize)bShouldResume=true;
			}
			if(bShouldResume){
				QString szTxt="The selected file\n"+szNewFileName+"\nalready exists.\n";
				szTxt+="It seems to be smaller in size than the offered one,\n";
				szTxt+="so it could be an incomplete file from a previous\n";
				szTxt+="unsuccesfull DCC Send.\n";
				szTxt+="Do you want to overwrite it with the offered one,\n";
				szTxt+="rename the new file or attempt to resume the previous DCC Send?\n";
				szTxt+="If you choose to rename the file,KVirc will auto-rename it to\n";
				szTxt+=szNewFileName+".rnm";
				m_lpFrm->m_bFocusManagingEnabled=false;
				int nResult=QMessageBox::information(m_lpFrm,"File exists",szTxt.data(),"Overwrite","Rename","Resume");
				m_lpFrm->m_bFocusManagingEnabled=true;
				if(nResult==0)return false; //overwrite , not resume
				else if(nResult==2)return true; //resume the prev dcc send
				else { //rename until get a good file name
					while(bExists){
						QFile aFile(szNewFileName);
						bExists=aFile.exists();
						if(bExists)szNewFileName+=".rnm";
					}
					return true;
				}
			} else {
				QString szTxt="The selected file\n"+szNewFileName+"\nalready exists.\n";
				szTxt+="Do you want to overwrite it with the offered one,\n";
				szTxt+="or rename the new file?\n";
				szTxt+="If you choose to rename the file,KVirc will auto-rename it to\n";
				szTxt+=szNewFileName+".rnm";
				m_lpFrm->m_bFocusManagingEnabled=false;
				int nResult=QMessageBox::information(m_lpFrm,"File exists",szTxt.data(),"Overwrite","Rename");
				m_lpFrm->m_bFocusManagingEnabled=true;
				if(nResult==0)return false; //overwrite , not resume
				else { //rename until get a good file name
					while(bExists){
						QFile aFile(szNewFileName);
						bExists=aFile.exists();
						if(bExists)szNewFileName+=".rnm";
					}
					return true;
				}
			}
		}
	}
	_debug_leavetrace("getDCCSaveFileName");
	return false; //unique file name
}

//============ handleDCCSend ============//

void KviCTCP::handleDCCSend(QString &szNick,QString &szMask,QString &szPara,QString &szAddr,QString &szPort,KviIrcToken *lpData)
{
	_debug_entertrace("handleDCCSend");
	if(!m_lpOpt->bIgnoreDCCSend){
		QString szFileSize=lpData->nextWord();
		if(szFileSize.isEmpty())szFileSize="unknown size";
		QString szReq=szNick+" ["+szMask+"]\nrequests a DCC SEND [";
		szReq+=szAddr+":"+szPort+"]\nfor file "+szPara+" "+szFileSize+" bytes long\nDo you accept?";
		if(!m_lpOpt->bAutoAcceptDCCSend){
			if(QMessageBox::information(m_lpFrm,"DCC SEND Request",szReq.data(),"&Yes","&No")==0){
				//Make the $home/Incoming directory (if is not there)
				QString szFileName;
				bool bResume=getDCCSaveFileName(szFileName,szPara,szFileSize);
				if(!szFileName.isEmpty()){
					QString szChatName="[DCC-GET]:-"+szNick+" "+szFileName;		
					KviSendWnd *lpWnd=m_lpFrm->createSendWnd(szChatName.data(),m_lpOpt->bMinimizeDCCSend);
					lpWnd->acceptSend(szNick.data(),szPort,szAddr,szFileName.data(),szFileSize,bResume);
				} else m_lpPar->m_lpCons->doFmtOutput(KVI_OUT_INTERNAL,"Refusing DCC SEND from %s [%s]",szNick.data(),szMask.data());	
			} else m_lpPar->m_lpCons->doFmtOutput(KVI_OUT_INTERNAL,"Refusing DCC SEND from %s [%s]",szNick.data(),szMask.data());
		} else {
			QString szFileName(getKVIrcHomeDirectory("Incoming"));
			QDir aDir(szFileName.data());
			if(!aDir.exists())aDir.mkdir(szFileName.data(),true);
			if(szPara[0] != '/')szFileName+='/';
			szFileName+=szPara;
			bool bExists=true;
			while(bExists){
				QFile aFile(szFileName.data());
				bExists=aFile.exists();
				if(bExists)szFileName+=".rnm";
			}
			m_lpPar->m_lpCons->doFmtOutput(KVI_OUT_INTERNAL,"Auto-accepting DCC SEND from %s [%s] for file [%s]",szNick.data(),szMask.data(),szFileName.data());
			QString szChatName="[DCC-GET]:-"+szNick+" "+szFileName;		
			KviSendWnd *lpWnd=m_lpFrm->createSendWnd(szChatName.data(),m_lpOpt->bMinimizeDCCSend);
			lpWnd->acceptSend(szNick.data(),szPort,szAddr,szFileName.data(),szFileSize);			
		}
	} else replyCTCPErrormsg(szNick,"DCC SEND protocol is currently disabled. (user request)");
	_debug_leavetrace("handleDCCSend");
}

//============ handleCTCPDCCRequest ============//

void KviCTCP::handleCTCPDCCRequest(QString &szNick,KviIrcToken *lpData,QString &szMask)
{
	_debug_entertrace("handleCTCPDCCRequest");
	
	QString szType=lpData->nextWord().upper();
	QString szPara=lpData->nextWord();
	QString szAddr=lpData->nextWord();
	QString szPort=lpData->nextWord();
	
	if(szType.isEmpty() || szPara.isEmpty() || szAddr.isEmpty() || szPort.isEmpty()){
			m_lpPar->topWin()->doFmtOutput(KVI_OUT_CTCP,"Malformed CTCP DCC from %s [%s]:[%s:%s:%s:%s]",
						szNick.data(),szMask.data(),szType.data(),szPara.data(),szAddr.data(),szPort.data());
	} else {
		//got a correct ctcp DCC
		if(szType=="CHAT")handleDCCChat(szNick,szMask,szPara,szAddr,szPort);
		else if(szType=="SEND")handleDCCSend(szNick,szMask,szPara,szAddr,szPort,lpData);
		else if(szType=="RESUME"){
			KviMdiChild *lpC=0;
			QString szFile=szPara.data();
			QString szNewPort=szAddr.data();
			QString szPosition=szPort.data();
			bool bOK=false;
			unsigned short int iPort=(unsigned short int)szNewPort.toUInt(&bOK);
			m_lpPar->topWin()->doFmtOutput(KVI_OUT_CTCP,
					"Received CTCP DCC RESUME from %s [ file :%s port :%s position :%s ]",
					szNick.data(),szFile.data(),szNewPort.data(),szPosition.data());
			if(bOK){
				for(lpC=m_lpFrm->m_lpMdi->m_lpChildList->first();lpC;lpC=m_lpFrm->m_lpMdi->m_lpChildList->next()){
					if(lpC->type()==KVI_WND_TYPE_SEND){
						KviSendWnd *lpW=(KviSendWnd*)lpC;
						if(!strcasecmp(lpW->m_szRemoteNick.data(),szNick.data())){
							if(lpW->m_iPort==iPort){ //got it
								if(lpW->m_bSending){ //look if we not sending something ourself
									uint iPosition=szPosition.toUInt(&bOK);
									if(bOK && (iPosition < lpW->m_iFileLength)){
										if(lpW->dccResumeForCurrentSend(iPosition)){
											m_lpSock->sendFmtData("PRIVMSG %s :%cDCC ACCEPT %s %s %s%c",
												szNick.data(),0x01,szFile.data(),szNewPort.data(),szPosition.data(),0x01);
										} else replyCTCPErrormsg(szNick,"DCC RESUME : File position requested is out of range.");
									} else replyCTCPErrormsg(szNick,"DCC RESUME : File position is out of range.");
									return;
								}
							}
						}
					}
				}
			}
			m_lpPar->topWin()->doFmtOutput(KVI_OUT_CTCP,
					"Refusiong DCC RESUME from %s [ file :%s port :%s position :%s ] : Transfer not initiated.",
					szNick.data(),szFile.data(),szNewPort.data(),szPosition.data());
			replyCTCPErrormsg(szNick,"DCC RESUME : Transfer not initiated.");
		} else if(szType=="ACCEPT"){
			KviMdiChild *lpC=0;
			QString szFile=szPara.data();
			QString szNewPort=szAddr.data();
			QString szPosition=szPort.data();
			bool bOK=false;
			unsigned short int iPort=(unsigned short int)szNewPort.toUInt(&bOK);
			if(bOK){
				for(lpC=m_lpFrm->m_lpMdi->m_lpChildList->first();lpC;lpC=m_lpFrm->m_lpMdi->m_lpChildList->next()){
					if(lpC->type()==KVI_WND_TYPE_SEND){
						KviSendWnd *lpW=(KviSendWnd*)lpC;
						if(!strcasecmp(lpW->m_szRemoteNick.data(),szNick.data())){
							if(lpW->m_iPort==iPort){ //got it
								if(!lpW->m_bSending){
									lpW->initiateResumeGet();
									return;
								}
							}
						}
					}
				}
			}
			m_lpPar->topWin()->doFmtOutput(KVI_OUT_CTCP,
					"Refusiong DCC ACCEPT from %s [ file :%s port :%s position :%s ] : No DCC RESUME Requested.",
					szNick.data(),szFile.data(),szNewPort.data(),szPosition.data());
			replyCTCPErrormsg(szNick,"DCC ACCEPT : No DCC RESUME Requested.");
		} else replyCTCPErrormsg(szNick,"DCC WHAT ??? [ Known DCC types: SEND,GET,RESUME,ACCEPT ]");
	}
	_debug_leavetrace("handleCTCPDCCRequest");
}

void KviCTCP::handleCTCPRequest(KviNick &nick,QString &target,KviIrcToken *lpData){
	//remove the 0x01
	lpData->szString.remove(0,1);
	lpData->szString.remove(lpData->szString.length()-1,1);
	//Get the CTCP type
	QString szCTCP=lpData->nextWord();
	//Get the params (if any)
	if(lpData->szString.isEmpty())lpData->szString="***";
	//Get the source nick mask
	QString szMask;
	nick.mask(szMask,1);

	target=target.stripWhiteSpace();
	szCTCP=szCTCP.upper();
	
	if(m_lpFrm->m_lpEventManager->lpEvent[KVI_Event_OnCtcpRequest]->bEnabled){
			QString szPara=nick.szNick.data();
			szPara+=" ";
			szPara+=szMask+" "+szCTCP+" "+lpData->szString;
			if(m_lpFrm->m_lpUserParser->executeEvent(m_lpFrm->m_lpConsole,
								m_lpFrm->m_lpEventManager->lpEvent[KVI_Event_OnCtcpRequest],
								szPara)==KVI_EventErr_Halt)return; //allow disabling the reply
 	}

	if(szCTCP=="ACTION"){ handleCTCPActionRequest(nick.szNick,target,lpData); return; }
	else if(szCTCP=="PAGE"){ handleCTCPPageRequest(nick.szNick,target,lpData); return; }

	//Start the timer anc check for flood
	if(m_lpOpt->iTimerCTCP>=1){
		if(!m_lpCTCPTimer->isActive())m_lpCTCPTimer->start(m_lpOpt->iTimerCTCP * 1000);
		m_iCTCPCount++;
	}
	if((m_iCTCPCount==m_lpOpt->iMaxCTCP)&&(m_lpOpt->iMaxCTCP>3)){
		m_lpPar->topWin()->doFmtOutput(KVI_OUT_FLOOD,"WARNING : CTCP flood detected. Last: CTCP %s from %s [%s]",
			szCTCP.data(),nick.szNick.data(),szMask.data());

		if(m_lpFrm->m_lpEventManager->lpEvent[KVI_Event_OnCtcpFlood]->bEnabled){
			QString szPara=nick.szNick.data();
			szPara+=" ";
			szPara+=szMask+" "+szCTCP+" "+lpData->szString;
			m_lpFrm->m_lpUserParser->executeEvent(m_lpFrm->m_lpConsole,
					m_lpFrm->m_lpEventManager->lpEvent[KVI_Event_OnCtcpFlood],szPara);
		}
	
	}
	if((m_iCTCPCount>m_lpOpt->iMaxCTCP)||(m_lpOpt->iTimerCTCP<1)){
		m_lpPar->m_lpCons->doFmtOutput(KVI_OUT_FLOOD,"Ignoring CTCP %s from %s [%s] [%s]",
			szCTCP.data(),nick.szNick.data(),szMask.data(),lpData->szString.data());
		return;
	}

	if((target[0] != '#')&&(target[0] != '&')){
		m_lpPar->m_lpCons->doFmtOutput(KVI_OUT_CTCP,"CTCP %s request from %s [%s] [%s]",
					szCTCP.data(),nick.szNick.data(),szMask.data(),lpData->szString.data());
	} else {
		m_lpPar->findWin(target.data())->doFmtOutput(KVI_OUT_CTCP,"Channel CTCP %s request from %s [%s] [%s]",
					szCTCP.data(),nick.szNick.data(),szMask.data(),lpData->szString.data());	
	}

	QString szReply;
	if(szCTCP=="PING"){
		replyCTCP(nick,"PING",lpData->szString.data());
	} else if(szCTCP=="VERSION"){
		szReply="Running ";
		szReply+=KVI_VERSION;
		QString szScript=m_lpOpt->szScriptVer.copy();
		m_lpFrm->m_lpUserParser->m_lpIdentifiers->substitute(m_lpFrm->m_lpConsole,szScript,0);
		if(!szScript.isEmpty()){
			szReply+=' ';
			szReply+=szScript;
		}
		replyCTCP(nick,"VERSION",szReply.data());
	} else if(szCTCP=="SOURCE"){
		szReply=KVI_CTCP_SOURCE_REPLY;
		replyCTCP(nick,"SOURCE",szReply.data());
	} else if(szCTCP=="TIME"){
		replyCTCP(nick,"TIME",QDateTime::currentDateTime().toString().data());
	} else if(szCTCP=="CLIENTINFO"){
		szReply=KVI_CLIENTINFO;
		szReply+=" - [Accepts:PING TIME VERSION CLIENTINFO SOURCE USERINFO FINGER DCC [CHAT SEND RESUME] ACTION PAGE]";
		replyCTCP(nick,"CLIENTINFO",szReply);
	} else if(szCTCP=="USERINFO"){
		if(m_lpOpt->szUserInfo.isEmpty())szReply=m_lpGlb->szUserName+" "+m_lpGlb->szRealName;
		else {
			szReply=m_lpOpt->szUserInfo.copy();
			m_lpFrm->m_lpUserParser->m_lpIdentifiers->substitute(m_lpFrm->m_lpConsole,szReply,0);
		}
		replyCTCP(nick,"USERINFO",szReply);
	} else if(szCTCP=="FINGER"){
		szReply=m_lpOpt->szFinger.copy();
		m_lpFrm->m_lpUserParser->m_lpIdentifiers->substitute(m_lpFrm->m_lpConsole,szReply,0);
		replyCTCP(nick,"FINGER",szReply);
	} else if(szCTCP=="DCC"){
		handleCTCPDCCRequest(nick.szNick,lpData,szMask);
	} else { //Can't handle it...
		replyCTCPErrormsg(nick.szNick,"Huh ??? [ Try CLIENTINFO for a list of CTCP types that this client can handle ]");
		m_lpPar->topWin()->doFmtOutput(KVI_OUT_CTCP,
			"%s: %s :Unknown CTCP type ... replying warning msg.(Huh???)",nick.szNick.data(),szCTCP.data());
 	}	
}

//============ replyCTCP ============//

void KviCTCP::replyCTCP(KviNick &nick,const char *szType,const char * szReply)
{
	_debug_entertrace("replyCTCP");
	m_lpSock->sendFmtData("NOTICE %s :%c%s %s%c",nick.szNick.data(),0x01,szType,szReply,0x01);
	_debug_leavetrace("replyCTCP");
}


void KviCTCP::resetCTCPCount(){
	m_lpCTCPTimer->stop();
	m_iCTCPCount=0;
}

void KviCTCP::handleCTCPReply(KviNick &nick,QString &target,KviIrcToken *lpData){
	lpData->szString.remove(0,1);
	lpData->szString.remove(lpData->szString.length()-1,1);
	QString szCTCP=lpData->nextWord();

	if(lpData->szString.isEmpty())lpData->szString="***";
	QString szMask;
	nick.mask(szMask,1);
	target=target.stripWhiteSpace();
	szCTCP=szCTCP.upper();

	if(m_lpFrm->m_lpEventManager->lpEvent[KVI_Event_OnCtcpReply]->bEnabled){
			QString szPara=nick.szNick.data();
			szPara+=" ";
			szPara+=szMask;
			szPara+=" ";
			szPara+=szCTCP;
			szPara+=" ";
			szPara+=lpData->szString.data();
			m_lpFrm->m_lpUserParser->executeEvent(m_lpFrm->m_lpConsole,
					m_lpFrm->m_lpEventManager->lpEvent[KVI_Event_OnCtcpReply],szPara);
	}

	if(szCTCP=="ACTION"){ handleCTCPActionRequest(nick.szNick,target,lpData); return; }
	else if(szCTCP=="PAGE"){ handleCTCPPageRequest(nick.szNick,target,lpData); return; }

	if(szCTCP != "PING"){
		m_lpPar->topWin()->doFmtOutput(KVI_OUT_CTCP,"CTCP %s reply from %s [%s]:%s",
					szCTCP.data(),nick.szNick.data(),szMask.data(),lpData->szString.data());
	} else {
		uint oldTime = (uint)(atoi(lpData->szString.data()));
		time_t curTime = time(0);
		uint elapsed = (uint)curTime - oldTime;
		m_lpPar->topWin()->doFmtOutput(KVI_OUT_CTCP,"CTCP PING reply from %s [%s]:%u seconds",
					nick.szNick.data(),szMask.data(),elapsed);
	}
}
#include "m_kvi_ctcp.moc"
