//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//


#define _KVI_DEBUG_CLASS_NAME_ "KviFrame"

#include "kvi_app.h"

#include <kfiledialog.h>
#include "kvi_ascii.h"
#include "kvi_dns.h"
#include "kvi_script.h"
#include "kvi_scrdlg.h"
#include "kvi_setup.h"
#include "kvi_about.h"
#include "kvi_translate.h"
#include "kvi_netscape.h"

pixmapTable KviFrame::pixTab[KVI_NUM_PIXMAPS]={
	{ KVI_OUT_INTERNAL  , "kvi_intrn.gif"  , "Internal messages"     },
	{ KVI_OUT_KVIRC     , "kvi_kvirc.gif"  , "KVirc version"         },
	{ KVI_OUT_ERROR     , "kvi_error.gif"  , "Errors"                },
	{ KVI_OUT_HELP      , "kvi_help.gif"   , "Help messages"         },
	{ KVI_OUT_MOTD      , "kvi_motd.gif"   , "Server's MOTD"         },
	{ KVI_OUT_QUIT      , "kvi_quit.gif"   , "Quit messages"         },
	{ KVI_OUT_KICK      , "kvi_kick.gif"   , "Channel kicks"         },
	{ KVI_OUT_INFO      , "kvi_info.gif"   , "Server's informations" },
	{ KVI_OUT_JOIN      , "kvi_join.gif"   , "Channel joins"         },
	{ KVI_OUT_PART      , "kvi_part.gif"   , "Channel parts"         },
	{ KVI_OUT_OP        , "kvi_op.gif"     , "Op messages"           },
	{ KVI_OUT_DEOP      , "kvi_deop.gif"   , "Deop messages"         },
	{ KVI_OUT_BAN       , "kvi_ban.gif"    , "Bans"                  },
	{ KVI_OUT_UNBAN     , "kvi_unban.gif"  , "Unbans"                },
	{ KVI_OUT_VOICE     , "kvi_voice.gif"  , "Voice mode changes"    },
	{ KVI_OUT_DEVOICE   , "kvi_devoi.gif"  , "Devoice mode changes"  },
	{ KVI_OUT_SERVER    , "kvi_serv.gif"   , "Server messages"       },
	{ KVI_OUT_CONNECT   , "kvi_conn.gif"   , "Connect"               },
	{ KVI_OUT_DISCONNECT, "kvi_disc.gif"   , "Disconnect"            },
	{ KVI_OUT_CLONE     , "kvi_clone.gif"  , "Clone warnings"        },
	{ KVI_OUT_CHANMODE  , "kvi_chmod.gif"  , "Channel modes"         },
	{ KVI_OUT_CTCP      , "kvi_ctcp.gif"   , "CTCP's"                },
	{ KVI_OUT_NICK      , "kvi_nick.gif"   , "Nick changes"          },
	{ KVI_OUT_PING      , "kvi_ping.gif"   , "Server pings"          },
	{ KVI_OUT_RPLERR    , "kvi_rpler.gif"  , "Server error replies"  },
	{ KVI_OUT_TOPIC     , "kvi_topic.gif"  , "Topic changes"         },
	{ KVI_OUT_OWN       , "kvi_own.gif"    , "Own channel text"      },
	{ KVI_OUT_ECHO      , "kvi_echo.gif"   , "Echo"                  },
	{ KVI_OUT_PRIVMSG   , "kvi_privm.gif"  , "Private messages"      },
	{ KVI_OUT_NOTICE    , "kvi_notic.gif"  , "Notices"               },
	{ KVI_OUT_RAW       , "kvi_raw.gif"    , "Raw messages"          },
	{ KVI_OUT_ACTION    , "kvi_act.gif"    , "Actions (CTCP)"        },
	{ KVI_OUT_FLOOD     , "kvi_flood.gif"  , "Flood warnings"        },
	{ KVI_OUT_WHO       , "kvi_who.gif"    , "Whois messages"        },
	{ KVI_OUT_TIME      , "kvi_time.gif"   , "Time"                  },
	{ KVI_OUT_STDOUT    , "kvi_stdo.gif"   , "Process stdout"        },
	{ KVI_OUT_STDERR    , "kvi_stde.gif"   , "Process stderr"        },
	{ KVI_OUT_STDIN     , "kvi_stdi.gif"   , "Process stdin"         },
	{ KVI_OUT_AWAY      , "kvi_away.gif"   , "Away message"          },
	{ KVI_OUT_BACK      , "kvi_back.gif"   , "Back message"          },
	{ KVI_OUT_DNS       , "kvi_dns.gif"    , "DNS query"             }
};

//============ KviFrame ============//
KviFrame::KviFrame()
{
	_debug_entertrace("KviFrame");

	m_lpSetup          = new KviSetup();
	m_lpSetup->setup();

	readSettings();
	createPixmaps();

	m_bFocusManagingEnabled=true;
	m_bRestarting=false;
	m_bUsingProxy=false;

	m_lpClr            = new KviMircColors(this); //creates the this->m_lpColor[16] colors!!!

	m_lpInt            = new KviInterface();
	m_lpUsr            = new KviUser();
	m_lpSrv            = new KviSrv();
	m_lpGlb            = new KviGlobals();
	m_lpOpt            = new KviOptions();
	m_lpUserParser     = new KviUserParser();
	m_lpAliasManager   = new KviAliasManager();
	m_lpEventManager   = new KviEventManager();
	m_lpPopupManager   = new KviPopupManager();
	m_lpAudio          = new KviAudio();
	m_lpTranslator     = new KviTranslator();
	m_lpNetscape       = new KviNetscapeController();
	m_lpDns            = new KviDns();
	m_lpAsciiPlayer    = new KviAsciiPlayer(this);
	connect(m_lpDns,SIGNAL(dnsDone(const char *)),this,SLOT(dnsQueryDone(const char *)));
	m_lpScriptManager  = new KviScriptManager(this);
	//set the translation file
	if(m_lpOpt->bUseTranslation){
		if(m_lpOpt->szTranslationFile=="[__none__]")m_lpOpt->bUseTranslation=false;
		else {
			if(!m_lpTranslator->setTranslationMap(m_lpOpt->szTranslationFile.data())){
				m_lpOpt->bUseTranslation=false;
				m_lpOpt->szTranslationFile="[__none__]";
			}
		}
	}

	connect(m_lpPopupManager->m_lpUserNickListPopup,SIGNAL(actionSelected(const char *)),this,SLOT(nickListUserSlot(const char *)));
	connect(m_lpPopupManager->m_lpUserChanPopup,SIGNAL(actionSelected(const char *)),this,SLOT(chanUserSlot(const char *)));
	connect(m_lpPopupManager->m_lpUserActionPopup,SIGNAL(actionSelected(const char *)),this,SLOT(slotUserAction(const char *)));
	connect(m_lpPopupManager->m_lpUserChatPopup,SIGNAL(actionSelected(const char *)),this,SLOT(chatUserSlot(const char *)));
	connect(m_lpPopupManager->m_lpUserQueryPopup,SIGNAL(actionSelected(const char *)),this,SLOT(queryUserSlot(const char *)));

	createStatusBar();
	createMdiManager();

	createNickListPopup();
	createNotifyListPopup();
	createStatusPopup();
	createChanPopup();
	createMenuBar();
	createToolBar();

	m_lpToolBar->setItemEnabled(2,false);	

	setCaption((m_lpOpt->szCaption.isEmpty() ? KVI_FRAME_CAPTION : m_lpOpt->szCaption.data()));
	setFocusPolicy(QWidget::StrongFocus);
	setFocusTargetRecursive(this,m_lpMdi);

	m_lpConsole        = new KviStatusWnd(m_lpMdi,this);
	m_lpMdi->addChild(m_lpConsole,"kvii_stat.gif");
	m_lpConsole->setGeometry(m_statusWndRect);
	m_lpConsole->m_bMaximized=m_bStatusWndMaximized;

	m_lpServerParser   = new KviServerParser(this);
	m_lpSock           = new KviIrcSocket(this);
	m_lpServerParser->activate(m_lpSock);
	m_lpUserParser->activate(this);

	m_iCurHostIdx=0;
	m_bCanReconnect=true;
	m_bReconnecting=false;
	m_iNextChildIdx=0;

	m_lpIsOnList       = new QList<int>;
	m_lpIsOnList->setAutoDelete(true);

	m_lpReconnectTimer = new QTimer();
	m_lpNotifyTimer    = new QTimer();
	m_lpUserHostTimer  = new QTimer();
	m_lpDestroyTimer   = new QTimer();
	connect(m_lpDestroyTimer,SIGNAL(timeout()),this,SLOT(destroySlot()));
	connect(m_lpNotifyTimer,SIGNAL(timeout()),this,SLOT(checkNotifyList()));
	connect(m_lpReconnectTimer,SIGNAL(timeout()),this,SLOT(reconnectionSlot()));
	connect(m_lpUserHostTimer,SIGNAL(timeout()),this,SLOT(doUserHost()));

	m_szNextUserHostQuery="";
	applyOptions();
	clearHostDescription();
	m_lpLastChildToDestroy=0;
	show();
	//call the Startup Event now...
	if(m_lpEventManager->lpEvent[KVI_Event_OnStartup]->bEnabled){
			QString szPara=KVI_VERSION;
			m_lpUserParser->executeEvent(m_lpConsole,m_lpEventManager->lpEvent[KVI_Event_OnStartup],szPara);
	}
	m_lpLinksDialog=0;
	_debug_leavetrace("KviFrame");
}
//============ ~KviFrame ============//
KviFrame::~KviFrame()
{
	_debug_entertrace("~KviFrame");
	if(!m_bRestarting){
		if(m_lpEventManager->lpEvent[KVI_Event_OnShutdown]->bEnabled){
				QString szPara=KVI_VERSION;
				m_lpUserParser->executeEvent(m_lpConsole,m_lpEventManager->lpEvent[KVI_Event_OnShutdown],szPara);
		}
	}
	_macro_kviApplication->processEvents();
	saveSettings();
	if(m_lpLinksDialog)delete m_lpLinksDialog;
	if(m_lpUserHostTimer->isActive())m_lpUserHostTimer->stop();
	while(!m_lpIsOnList->isEmpty())m_lpIsOnList->removeLast();
	delete m_lpIsOnList;
	if(m_lpDestroyTimer->isActive())m_lpDestroyTimer->stop();
	if(m_lpReconnectTimer->isActive())m_lpReconnectTimer->stop();
	if(m_lpNotifyTimer->isActive())m_lpNotifyTimer->stop();
	delete m_lpUserHostTimer;
	delete m_lpNotifyTimer;
	delete m_lpAsciiPlayer;
	delete m_lpSock; //first kill the socket...
	delete m_lpStatusBar;
	delete m_lpMenuBar;
		delete m_lpSettingsPopup;
		delete m_lpMainPopup;
		delete m_lpActionsPopup;
		delete m_lpUModePopup;
	delete m_lpNotifyListPopup;
	delete m_lpNickListPopup;
		delete m_lpControlPopup;
		delete m_lpCTCPPopup;
		delete m_lpDCCPopup;
		delete m_lpInfoPopup;
		delete m_lpToolPopup;
	delete m_lpStatusPopup;
		delete m_lpStatsPopup;
	delete m_lpPopupManager;
	delete m_lpChanPopup;
	delete m_lpToolBar;
	delete m_lpMdi;
	delete m_lpInt;
	delete m_lpUsr;
	delete m_lpSrv;
	delete m_lpGlb;
	delete m_lpOpt;
	delete m_lpServerParser;
	delete m_lpAliasManager;
	delete m_lpEventManager;
	delete m_lpAudio;
	delete m_lpTranslator;
	delete m_lpNetscape;
	delete m_lpSetup;
	delete m_lpClr; //destroys the colors!!!!!!!
	delete m_lpDns;
	destroyPixmaps();
	delete m_lpScriptManager;
	delete m_lpUserParser;
	_debug_leavetrace("~KviFrame");
}
//============ doLinksDialog ============//
void KviFrame::doLinksDialog()
{
	_debug_entertrace("doLinksDialog");
	if(!m_lpLinksDialog){
		m_lpLinksDialog=new KviLinksDialog(this);
	}
	m_lpToolBar->setItemEnabled(2,false);	
	_debug_leavetrace("doLinksDialog");
}
//============ linksDialogClosed ============//
void KviFrame::linksDialogClosed()
{
	_debug_entertrace("linksDialogClosed");
	m_lpLinksDialog=0;
	if(m_lpSock->m_bConnected)m_lpToolBar->setItemEnabled(2,true);	
	_debug_leavetrace("linksDialogClosed");
}
//============ formatNickOutputString ============//
void KviFrame::formatNickOutputString(QString &szResult,QString &szNick)
{
	_debug_entertrace("formatNickOutputString");
	if(m_lpOpt->bInverseNick)szResult="<\x016";
	else szResult="<";
	szResult+=szNick;
	if(m_lpOpt->bInverseNick)szResult+="\x0f>";
	else szResult+=">";
	if(m_lpOpt->bAlignText){
		if(szResult.length()<12){
			szResult.setExpand(11,(m_lpOpt->bPrefixMsg ? ':' : ' '));
		} else szResult+=(m_lpOpt->bPrefixMsg ? ':' : ' ');
	} else {
		szResult+=(m_lpOpt->bPrefixMsg ? ':' : ' ');
	}
	_debug_leavetrace("formatNickOutputString");
}
//============ saveSettings ============//
void KviFrame::saveSettings()
{
	_debug_entertrace("saveSettings");
	_macro_kviConfig->setGroup("KviFrame");
	_macro_kviConfig->writeEntry("geometry", KWM::getProperties(winId()));
	_macro_kviConfig->writeEntry("StatusWnd",m_lpConsole->rect());
	_macro_kviConfig->writeEntry("StatusWndMax",m_lpConsole->m_bMaximized);
	_debug_leavetrace("saveSettings");
}
//============ readSettings ============//
void KviFrame::readSettings()
{
	_debug_entertrace("readSettings");
	KConfig *lpCfg=_macro_kviConfig;
	lpCfg->setGroup("KviFrame");
    if(!lpCfg->hasKey("geometry"))return;
    // Use KWM for window properties
    QString geom = lpCfg->readEntry("geometry");
    if(!geom.isEmpty())setGeometry(KWM::setProperties(winId(),geom));
	else setGeometry(0,0,600,500);
	QRect rct(0,0,400,300);
	m_statusWndRect= lpCfg->readRectEntry("StatusWnd",&rct);
	m_bStatusWndMaximized=lpCfg->readNumEntry("StatusWndMax",false);
	_debug_leavetrace("readSettings");
}
//============ viewDoubleClicked ============//
void KviFrame::viewDoubleClicked(QString &szString)
{
	_debug_entertrace("viewDoubleClicked");
	if(m_lpOpt->bRunBrowserOnUrl){
		//check for a url in the selectioin
		int idxUrl=szString.find("http:/");
		if(idxUrl<0){
			idxUrl=szString.find("ftp:/");
			if(idxUrl<0){
				idxUrl=szString.find("www");
				if(idxUrl<0){
					idxUrl=szString.find("ftp");
					if(idxUrl<0){
						idxUrl=szString.find("file:/");
					}
				}
			}
		}
		if(idxUrl>=0){
			szString.remove(0,idxUrl);
			if(m_lpOpt->bUseNetscape){
				if(!m_lpNetscape->openURL(szString.data())){
					m_lpConsole->doOutput(KVI_OUT_ERROR,"[URLCatcher]:Unable to execute the openURL command on the remote Netscape.");
				}
			} else {
				if(!m_lpOpt->szBrowserCmdLine.isEmpty()){
					KProcess *proc=new KProcess();
					QString szCmd=m_lpOpt->szBrowserCmdLine.copy();
					int idxPercent=szCmd.find('%');
					if(idxPercent != -1){
						szCmd.replace(idxPercent,1,szString.data());
						QString szArg;
						while(!szCmd.isEmpty()){
							int idx=szCmd.find(' ');
							if(idx!=-1){
								szArg=szCmd.left(idx);
								szCmd.remove(0,idx+1);
							} else {
								szArg=szCmd.copy();
								szCmd="";
							}
							(*proc) << szArg.data();
						}
						if(!proc->start(KProcess::DontCare,KProcess::NoCommunication)){
							m_lpConsole->doFmtOutput(KVI_OUT_ERROR,"[URLCatcher]:Unable to run the requested browser [%s]",m_lpOpt->szBrowserCmdLine.data());
						}
					} else {
						m_lpConsole->doOutput(KVI_OUT_ERROR,"[URLCatcher]:Browser commandline syntax error.The commandline must contain a '%'");
					}
					delete proc;
				} else {
					m_lpConsole->doOutput(KVI_OUT_ERROR,"[URLCatcher]:Browser commandline empty.Don't know wchih browser to run.");
				}
			}
		}	
	}	
	_debug_leavetrace("viewDoubleClicked");
}
//============ applyOptions ============//
void KviFrame::applyOptions()
{
	_debug_entertrace("applyOptions");
	m_lpMdi->applyOptions();
	_debug_leavetrace("applyOptions");
}
//============ createMdiManager ============//
void KviFrame::createMdiManager()
{
	_debug_entertrace("createMdiManager");
	m_lpMdi = new KviMdiManager(this);
	setView(m_lpMdi);
	_debug_leavetrace("createMdiManager");
}
//============ createPixmaps ============//
void KviFrame::createPixmaps()
{
	_debug_entertrace("createPixmaps");
	KIconLoader *lpI=_macro_kviIconLoader;
	QPixmap pixmappa;
	for(int i=0;i<KVI_NUM_PIXMAPS;i++){
		pixmappa=lpI->loadIcon(pixTab[i].szPixName);
		if(!pixmappa.isNull())m_lpPixmap[pixTab[i].index]=new QPixmap(pixmappa);
		else _debug_fatal("Cannot find the base pixmaps...did you forget make install?");
	}
	_debug_leavetrace("createPixmaps");
}
//============ destroyPixmaps ============//
void KviFrame::destroyPixmaps()
{
	_debug_entertrace("destroyPixmaps");
	for(int i=0;i<KVI_NUM_PIXMAPS;i++)delete m_lpPixmap[i];
	_debug_leavetrace("destroyPixmaps");
}
//============ createChanWnd ============//
KviChanWnd * KviFrame::createChanWnd(const char *szName)
{
	_debug_entertrace("createChanWnd");
	m_iNextChildIdx++;
	KviChanWnd *lpW=new KviChanWnd(m_lpMdi,this,szName,m_iNextChildIdx);
	m_lpMdi->addChild(lpW,"kvii_chan.gif");
	_debug_leavetrace("createChanWnd");
	return lpW;
}
//============ createChatWnd ============//
KviChatWnd * KviFrame::createChatWnd(const char *szName)
{
	_debug_entertrace("createChatWnd");
	m_iNextChildIdx++;
	KviChatWnd *lpW=new KviChatWnd(m_lpMdi,this,szName,m_iNextChildIdx);
	m_lpMdi->addChild(lpW,"kvii_chat.gif");
	_debug_leavetrace("createChatWnd");
	return lpW;
}
//============ createQueryWnd ============//
KviQueryWnd * KviFrame::createQueryWnd(const char *szName)
{
	_debug_entertrace("createQueryWnd");
	m_iNextChildIdx++;
	KviQueryWnd *lpW=new KviQueryWnd(m_lpMdi,this,szName,m_iNextChildIdx);
	m_lpMdi->addChild(lpW,"kvii_qery.gif");
	_debug_leavetrace("createQueryWnd");
	return lpW;
}
//============ createSendWnd ============//
KviSendWnd * KviFrame::createSendWnd(const char *szName,bool bMinimized)
{
	_debug_entertrace("createSendWnd");
	m_iNextChildIdx++;
	KviSendWnd *lpW=new KviSendWnd(m_lpMdi,this,szName,m_iNextChildIdx);
	if(bMinimized)m_lpMdi->addChildMinimized(lpW,"kvii_send.gif");
	else m_lpMdi->addChild(lpW,"kvii_send.gif");
	_debug_leavetrace("createSendWnd");
	return lpW;
}
//============ createUserWnd ============//
KviUserWnd * KviFrame::createUserWnd(const char *szName,bool bMinimized,bool bHasInput,
								const char *szImage,QRect *geom)
{
	_debug_entertrace("createUserWnd");
	m_iNextChildIdx++;
	KviUserWnd *lpW=new KviUserWnd(m_lpMdi,this,bHasInput,szName,m_iNextChildIdx);
	bool bGeom=(geom ? false : true);
	if(geom)lpW->setGeometry((*geom));
	if(szImage)lpW->m_lpOutput->setUserBackground(szImage);
	if(bMinimized)m_lpMdi->addChildMinimized(lpW,"kvii_usrwnd.gif",bGeom);
	else m_lpMdi->addChild(lpW,"kvii_usrwnd.gif",bGeom);
	_debug_leavetrace("createUserWnd");
	return lpW;
}
//============ createStatusPopup ============//
void KviFrame::createStatusPopup()
{
	_debug_entertrace("createStatusPopup");
	m_lpStatusPopup = new QPopupMenu();
	m_lpStatsPopup  = new QPopupMenu();
	KIconLoader *lpI=_macro_kviIconLoader;
	m_lpStatusPopup->insertItem(lpI->loadIcon("kvi_motd.gif"),"View Message of the day",this,SLOT(slotStatusMotd()));
	m_lpStatusPopup->insertItem(lpI->loadIcon("kvi_kvirc.gif"),"Query server version",this,SLOT(slotStatusVersion()));
	m_lpStatusPopup->insertItem(lpI->loadIcon("kvi_time.gif"),"Query server's local time",this,SLOT(slotStatusTime()));
	m_lpStatusPopup->insertItem(lpI->loadIcon("kvi_info.gif"),"Query server's info",this,SLOT(slotStatusInfo()));
	m_lpStatusPopup->insertItem(lpI->loadIcon("kvi_invite.gif"),"Query admininstator e-mail",this,SLOT(slotStatusAdmin()));
	m_lpStatusPopup->insertItem(lpI->loadIcon("kvii_usr.gif"),"Query users list",this,SLOT(slotStatusUsers()));
	m_lpStatusPopup->insertItem(lpI->loadIcon("kvi_info.gif"),"Query users statistics",this,SLOT(slotStatusLusers()));
	m_lpStatusPopup->insertItem(lpI->loadIcon("kvi_links.gif"),"Query links",this,SLOT(doLinksDialog()));
	
	m_lpStatsPopup->insertItem(lpI->loadIcon("kvi_echo.gif"),"Query registered servers",this,SLOT(slotStatusStatsC()));
	m_lpStatsPopup->insertItem(lpI->loadIcon("kvi_kline.gif"),"Query K-Lines",this,SLOT(slotStatusStatsK()));
	m_lpStatsPopup->insertItem(lpI->loadIcon("kvi_time.gif"),"Query server's live time",this,SLOT(slotStatusStatsU()));
	m_lpStatsPopup->insertItem(lpI->loadIcon("kvi_echo.gif"),"Query commands supported",this,SLOT(slotStatusStatsM()));

	m_lpStatusPopup->insertItem(lpI->loadIcon("kvi_info.gif"),"Stats",m_lpStatsPopup);
	_debug_leavetrace("createStatusPopup");
}
//============ Status Slots ============//
void KviFrame::slotStatusStatsM() {statusSlot("STATS m");}
void KviFrame::slotStatusStatsU() {statusSlot("STATS u");}
void KviFrame::slotStatusStatsK() {statusSlot("STATS k");}
void KviFrame::slotStatusStatsC() {statusSlot("STATS c");}
//void KviFrame::slotStatusLinks()  {statusSlot("LINKS");}
void KviFrame::slotStatusLusers() {statusSlot("LUSERS");}
void KviFrame::slotStatusUsers()  {statusSlot("USERS");}
void KviFrame::slotStatusAdmin()  {statusSlot("ADMIN");}
void KviFrame::slotStatusInfo()   {statusSlot("INFO");}
void KviFrame::slotStatusMotd()   {statusSlot("MOTD");}
void KviFrame::slotStatusVersion(){statusSlot("VERSION");}
void KviFrame::slotStatusTime()   {statusSlot("TIME");}
//============ statusSlot ============//
void KviFrame::statusSlot(const char *sendData)
{
	_debug_entertrace("statusSlot");
	if(m_lpSock->m_bConnected)m_lpSock->sendData(sendData);
	_debug_leavetrace("statusSlot");
}
//============ createNotifyListPopup ============//
void KviFrame::createNotifyListPopup()
{
	_debug_entertrace("createNotifyListPopup");
	m_lpNotifyListPopup = new QPopupMenu();
	KIconLoader *lpI=_macro_kviIconLoader;
	m_lpNotifyListPopup->insertItem(lpI->loadIcon("kvi_who.gif"),"Whois",this,SLOT(slotNotifyWhois()));
	m_lpNotifyListPopup->insertSeparator();
	m_lpNotifyListPopup->insertItem(lpI->loadIcon("kvi_ping.gif"),"Ping",this,SLOT(slotNotifyPing()));
	m_lpNotifyListPopup->insertItem(lpI->loadIcon("kvi_kvirc.gif"),"Version",this,SLOT(slotNotifyVersion()));
	m_lpNotifyListPopup->insertSeparator();
	m_lpNotifyListPopup->insertItem(lpI->loadIcon("kvii_qery.gif"),"Query",this,SLOT(slotNotifyQuery()));
	m_lpNotifyListPopup->insertSeparator();
	m_lpNotifyListPopup->insertItem(lpI->loadIcon("kvii_chat.gif"),"DCC Chat",this,SLOT(slotNotifyDCCChat()));
	m_lpNotifyListPopup->insertItem(lpI->loadIcon("kvii_send.gif"),"DCC Send",this,SLOT(slotNotifyDCCSend()));
	_debug_leavetrace("createNotifyListPopup");
}
//============ createNickListPopup ============//
void KviFrame::createNickListPopup()
{
	_debug_entertrace("createNickListPopup");
	m_lpNickListPopup = new QPopupMenu();
	m_lpControlPopup  = new QPopupMenu();
	m_lpCTCPPopup     = new QPopupMenu();
	m_lpDCCPopup      = new QPopupMenu();
	m_lpInfoPopup     = new QPopupMenu();
	m_lpToolPopup     = new QPopupMenu();
	KIconLoader *lpI=_macro_kviIconLoader;
	m_lpControlPopup->insertItem(lpI->loadIcon("kvi_op.gif"),"Op",this,SLOT(slotOp()));
	m_lpControlPopup->insertItem(lpI->loadIcon("kvi_deop.gif"),"Deop",this,SLOT(slotDeop()));
	m_lpControlPopup->insertItem(lpI->loadIcon("kvi_voice.gif"),"Voice",this,SLOT(slotVoice()));
	m_lpControlPopup->insertItem(lpI->loadIcon("kvi_devoi.gif"),"Devoice",this,SLOT(slotDevoice()));
	m_lpControlPopup->insertItem(lpI->loadIcon("kvi_kick.gif"),"Kick",this,SLOT(slotKick()));
	m_lpControlPopup->insertItem(lpI->loadIcon("kvi_ban.gif"),"Kick & ban",this,SLOT(slotKBan()));

	m_lpCTCPPopup->insertItem(lpI->loadIcon("kvi_ping.gif"),"Ping",this,SLOT(slotPing()));
	m_lpCTCPPopup->insertItem(lpI->loadIcon("kvi_kvirc.gif"),"Version",this,SLOT(slotVersion()));
	m_lpCTCPPopup->insertItem(lpI->loadIcon("kvi_time.gif"),"Time",this,SLOT(slotTime()));
	m_lpCTCPPopup->insertItem(lpI->loadIcon("kvi_echo.gif"),"Clientinfo",this,SLOT(slotClientinfo()));
	m_lpCTCPPopup->insertItem(lpI->loadIcon("kvi_who.gif"),"Userinfo",this,SLOT(slotUserinfo()));
	m_lpCTCPPopup->insertItem(lpI->loadIcon("kvi_echo.gif"),"Finger",this,SLOT(slotFinger()));

	m_lpInfoPopup->insertItem(lpI->loadIcon("kvi_who.gif"),"Whois",this,SLOT(slotWhois()));

	m_lpInfoPopup->insertItem(lpI->loadIcon("kvi_who.gif"),"Whois with idle time",this,SLOT(slotWhoWho()));
	m_lpInfoPopup->insertItem(lpI->loadIcon("kvi_who.gif"),"Whowas",this,SLOT(slotWhowas()));
	m_lpInfoPopup->insertItem(lpI->loadIcon("kvi_echo.gif"),"Userlist entry",this,SLOT(slotUserlist()));

	m_lpDCCPopup->insertItem(lpI->loadIcon("kvii_chat.gif"),"Chat",this,SLOT(slotChat()));
	m_lpDCCPopup->insertItem(lpI->loadIcon("kvii_send.gif"),"Send",this,SLOT(slotDCCSend()));

	m_lpToolPopup->insertItem(lpI->loadIcon("kvi_clone.gif"),"Clonescan",this,SLOT(slotClonescan()));

	m_lpNickListPopup->insertItem(lpI->loadIcon("kvi_own.gif"),"Control...",m_lpControlPopup,1);
	m_lpNickListPopup->insertItem(lpI->loadIcon("kvi_ctcp.gif"),"Ctcp...",m_lpCTCPPopup,2);
	m_lpNickListPopup->insertItem(lpI->loadIcon("kvi_conn.gif"),"Dcc...",m_lpDCCPopup,3);
	m_lpNickListPopup->insertSeparator();
	m_lpNickListPopup->insertItem(lpI->loadIcon("kvi_info.gif"),"Info...",m_lpInfoPopup,4);
	m_lpNickListPopup->insertSeparator();
	m_lpNickListPopup->insertItem(lpI->loadIcon("kvi_raw.gif"),"Tools...",m_lpToolPopup,5);
	m_lpNickListPopup->insertSeparator();
	m_lpNickListPopup->insertItem(lpI->loadIcon("kvii_qery.gif"),"Query",this,SLOT(slotQuery()),0,6);
	m_lpNickListPopup->insertSeparator();
	m_lpNickListPopup->insertItem(lpI->loadIcon("kvi_echo.gif"),m_lpPopupManager->szNickListPopup.data(),
														m_lpPopupManager->m_lpUserNickListPopup,7);

	_debug_leavetrace("createNickListPopup");
}
//============ createChanPopup ============//
void KviFrame::createChanPopup()
{
	_debug_entertrace("createChanPopup");
	m_lpChanPopup=new QPopupMenu();
	KIconLoader *lpI=_macro_kviIconLoader;
	m_lpChanPopup->insertItem(lpI->loadIcon("kvi_kvirc.gif"),"KVirc",this,SLOT(slotChanKvirc()),0,1);
	m_lpChanPopup->insertSeparator();
//	m_lpChanPopup->insertItem(lpI->loadIcon("kvi_chmod.gif"),"Edit chan settings",this,SLOT(slotChanEdit()),0,2);
	m_lpChanPopup->insertItem(lpI->loadIcon("kvi_ban.gif"),"Edit ban list",this,SLOT(slotChanBanlist()),0,3);
	m_lpChanPopup->insertItem(lpI->loadIcon("kvi_who.gif"),"View userlist",this,SLOT(slotChanUserlist()),0,4);
	m_lpChanPopup->insertItem(lpI->loadIcon("kvi_clone.gif"),"Clonescan",this,SLOT(slotChanClonescan()),0,5);
	m_lpChanPopup->insertSeparator();
	m_lpChanPopup->insertItem(lpI->loadIcon("kvi_part.gif"),"Part channel",this,SLOT(slotChanPart()),0,6);
	m_lpChanPopup->insertSeparator();
	m_lpChanPopup->insertItem(lpI->loadIcon("kvi_echo.gif"),m_lpPopupManager->szChanPopup.data(),
															m_lpPopupManager->m_lpUserChanPopup,7);
	_debug_leavetrace("createChanPopup");
}
//============ requestChanPopup ============//
void KviFrame::requestChanPopup(QPoint pnt)
{
	_debug_entertrace("requestChanPopup");
	if(m_lpGlb->lpCurrentChan)m_lpChanPopup->popup(pnt);
	_debug_leavetrace("requestChanPopup");
}
//============ requestChatPopup ============//
void KviFrame::requestChatPopup(QPoint pnt)
{
	_debug_entertrace("requestChatPopup");
	if(m_lpGlb->lpCurrentChat)m_lpPopupManager->m_lpUserChatPopup->popup(pnt);
	_debug_leavetrace("requestChatPopup");
}
//============ requestQueryPopup ============//
void KviFrame::requestQueryPopup(QPoint pnt)
{
	_debug_entertrace("requestQueryPopup");
	if(m_lpGlb->lpCurrentQuery)m_lpPopupManager->m_lpUserQueryPopup->popup(pnt);
	_debug_leavetrace("requestQueryPopup");
}
//============ requestStatusPopup ============//
void KviFrame::requestStatusPopup(QPoint pnt)
{
	_debug_entertrace("requestStatusPopup");
	if(m_lpSock->m_bConnected){
		m_lpStatusPopup->popup(pnt);
	} else {
		m_lpMainPopup->popup(pnt);
	}
	_debug_leavetrace("requestStatusPopup");
}
void KviFrame::slotOp(){        nickListSlot("/OP");}
void KviFrame::slotDeop(){      nickListSlot("/DEOP");}
void KviFrame::slotVoice(){     nickListSlot("/VOICE");}
void KviFrame::slotDevoice(){   nickListSlot("/DEVOICE");}
void KviFrame::slotQuery(){     nickListSlot("/QUERY");}
void KviFrame::slotPing(){      nickListSlot("/CTCP PING");}
void KviFrame::slotChat(){      nickListSlot("/CHAT");}
void KviFrame::slotVersion(){   nickListSlot("/CTCP VERSION");}
void KviFrame::slotTime(){      nickListSlot("/CTCP TIME");}
void KviFrame::slotClientinfo(){nickListSlot("/CTCP CLIENTINFO");}
void KviFrame::slotUserinfo(){  nickListSlot("/CTCP USERINFO");}
void KviFrame::slotFinger(){    nickListSlot("/CTCP FINGER");}
void KviFrame::slotWhois(){     nickListSlot("/WHOIS");}
void KviFrame::slotKick(){      nickListSlot("/KICK");}
void KviFrame::slotDCCSend(){   nickListSlot("/SEND");}
void KviFrame::slotKBan(){      nickListSlot("/KBAN");}
void KviFrame::slotWhowas(){    nickListSlot("/WHOWAS");}
void KviFrame::slotWhoWho(){    nickListSlot("/WHOIS -i");}
void KviFrame::slotUserlist(){  nickListSlot("/USERLIST");}
void KviFrame::slotClonescan(){ nickListSlot("/CLONESCAN");}
void KviFrame::slotNotifyWhois(){   notifyListSlot("/WHOIS");}
void KviFrame::slotNotifyPing(){    notifyListSlot("/CTCP PING");}
void KviFrame::slotNotifyDCCChat(){ notifyListSlot("/CHAT");}
void KviFrame::slotNotifyDCCSend(){ notifyListSlot("/SEND");}
void KviFrame::slotNotifyVersion(){ notifyListSlot("/CTCP VERSION");}
void KviFrame::slotNotifyQuery(){   notifyListSlot("/QUERY");}
//============ slotChanClonescan ============//
void KviFrame::slotChanClonescan()
{
	_debug_entertrace("slotChanClonescan");
	QString szToDo="/CLONESCAN";
	if(m_lpGlb->lpCurrentChan)m_lpUserParser->parseUserInput(m_lpGlb->lpCurrentChan,KVI_WND_TYPE_CHAN,m_lpGlb->lpCurrentChan->name(),szToDo);
	_debug_leavetrace("slotChanClonescan");
}
//============ slotChanKvirc ============//
void KviFrame::slotChanKvirc()
{
	_debug_entertrace("slotChanKvirc");
	QString szToDo="$_version";
	if(m_lpGlb->lpCurrentChan)m_lpUserParser->parseUserInput(m_lpGlb->lpCurrentChan,KVI_WND_TYPE_CHAN,m_lpGlb->lpCurrentChan->name(),szToDo);
	_debug_leavetrace("slotChanKvirc");
}
//============ slotChanUserlist ============//
void KviFrame::slotChanUserlist()
{
	_debug_entertrace("slotChanUserlist");
	if(m_lpGlb->lpCurrentChan)m_lpGlb->lpCurrentChan->doUserList();
	_debug_leavetrace("slotChanUserlist");
}
//============ slotChanPart ============//
void KviFrame::slotChanPart()
{
	_debug_entertrace("slotChanPart");
	if(m_lpGlb->lpCurrentChan)m_lpGlb->lpCurrentChan->slotClose();
	_debug_leavetrace("slotChanPart");
}
//============ slotChanBanlist ============//
void KviFrame::slotChanBanlist()
{
	_debug_entertrace("slotChanBanlist");
	m_bFocusManagingEnabled=false;
	if(m_lpGlb->lpCurrentChan)m_lpGlb->lpCurrentChan->doBanDialog();
	m_bFocusManagingEnabled=true;
	_debug_leavetrace("slotChanBanlist");
}

//============ nickListSlot ============//
void KviFrame::nickListSlot(const char *szCommand){
	if(!m_lpGlb->szCurrentSelection.isEmpty() && m_lpGlb->lpCurrentChan){
		QString szToDo=szCommand;
		szToDo+=' ';
		szToDo+=m_lpGlb->szCurrentSelection;
		m_lpUserParser->parseUserInput(m_lpGlb->lpCurrentChan,KVI_WND_TYPE_CHAN,m_lpGlb->lpCurrentChan->name(),szToDo);
	}
}
//============ nickListUserSlot ============//
void KviFrame::nickListUserSlot(const char *szCommand)
{
	_debug_entertrace("nickListUserSlot");
	if(!m_lpGlb->szCurrentSelection.isEmpty() && m_lpGlb->lpCurrentChan){
		QString szToDo=szCommand;
		m_lpUserParser->executePopup(m_lpGlb->lpCurrentChan,szToDo);
	}
	_debug_leavetrace("nickListUserSlot");
}
//=========== chanUserSlot =============//
void KviFrame::chanUserSlot(const char *szCommand)
{
	_debug_entertrace("chanUserSlot");
	if(m_lpGlb->lpCurrentChan){
		QString szToDo=szCommand;
		m_lpUserParser->executePopup(m_lpGlb->lpCurrentChan,szToDo);
	}
	_debug_leavetrace("chanUserSlot");
}
//=========== chatUserSlot =============//
void KviFrame::chatUserSlot(const char *szCommand)
{
	_debug_entertrace("chatUserSlot");
	if(m_lpGlb->lpCurrentChat){
		QString szToDo=szCommand;
		m_lpUserParser->executePopup(m_lpGlb->lpCurrentChat,szToDo);
	}
	_debug_leavetrace("chatUserSlot");
}
//=========== queryUserSlot =============//
void KviFrame::queryUserSlot(const char *szCommand)
{
	_debug_entertrace("queryUserSlot");
	if(m_lpGlb->lpCurrentQuery){
		QString szToDo=szCommand;
		m_lpUserParser->executePopup(m_lpGlb->lpCurrentQuery,szToDo);
	}
	_debug_leavetrace("queryUserSlot");
}
//============ slotUserAction ============//
void KviFrame::slotUserAction(const char *szAction)
{
	_debug_entertrace("slotUserAction");
	QString szToDo=szAction;
	m_lpUserParser->executePopup(m_lpConsole,szToDo);
	_debug_leavetrace("slotUserAction");
}
//============ notifyListSlot ============//
void KviFrame::notifyListSlot(const char *szCommand){
	if(!m_lpGlb->szCurrentNotifySelection.isEmpty()){
		QString szToDo=szCommand;
		szToDo+=' ';
		szToDo+=m_lpGlb->szCurrentNotifySelection;
		m_lpUserParser->parseUserInput(m_lpConsole,KVI_WND_TYPE_STATUS,KVI_STATUS_WND_NAME,szToDo);
	}
}
//============ createMenuBar ============//
void KviFrame::createMenuBar()
{
	_debug_entertrace("createMenuBar");
	m_lpMenuBar = new KMenuBar(this,"KviMenuBar");
	KIconLoader *lpI=_macro_kviIconLoader;
	m_lpSettingsPopup = new QPopupMenu();
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_usr.gif"),"User...",this,SLOT(doUserDialog()));
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_srv.gif"),"Servers...",this,SLOT(doServerDialog()));
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_opt.gif"),"Misc options...",this,SLOT(doOptionsDialog()));
	m_lpSettingsPopup->insertSeparator();
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_int.gif"),"Interface...",this,SLOT(doInterfaceDialog()));
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_clr.gif"),"Irc colors...",this,SLOT(doIrcColorsDialog()));
	m_lpSettingsPopup->insertSeparator();
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_alias.gif"),"Aliases...",this,SLOT(doAliasDialog()));
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_event.gif"),"Events...",this,SLOT(doEventDialog()));
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_pop.gif"),"Popups...",this,SLOT(doPopupDialog()));
	m_lpSettingsPopup->insertSeparator();
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_save.gif"),"Save settings",this,SLOT(saveAllOptions()));
	m_lpSettingsPopup->insertSeparator();
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_load.gif"),"Load a script",this,SLOT(loadScript()));
	m_lpSettingsPopup->insertItem(lpI->loadIcon("kvii_save.gif"),"Save current script",this,SLOT(saveScript()));
	
	m_lpMainPopup = new QPopupMenu();
	m_lpMainPopup->insertItem(lpI->loadIcon("kvii_conn.gif"),"Connect",this,SLOT(connectionSlot()));
	m_lpMainPopup->insertSeparator();
	m_lpMainPopup->insertItem(lpI->loadIcon("kvi_kvirc.gif"),"New KVirc window",this,SLOT(newWindow()));
	m_lpMainPopup->insertSeparator();
	m_lpMainPopup->insertItem(lpI->loadIcon("kvi_disc.gif"),"Quit",_macro_kviApplication,SLOT(quit()));
	m_lpMainPopup->setId(0,0);

	m_lpUModePopup=new QPopupMenu();
	m_lpUModePopup->insertItem(lpI->loadIcon("kvi_echo.gif"),"Invisible",this,SLOT(slotUModeI()));
	m_lpUModePopup->insertItem(lpI->loadIcon("kvi_echo.gif"),"Server notices",this,SLOT(slotUModeS()));
	m_lpUModePopup->insertItem(lpI->loadIcon("kvi_echo.gif"),"Wallops",this,SLOT(slotUModeW()));

	m_lpActionsPopup=new QPopupMenu();
	m_lpActionsPopup->insertItem(lpI->loadIcon("kvii_srv.gif"),"Server info",m_lpStatusPopup);
	m_lpActionsPopup->insertSeparator();
	m_lpActionsPopup->insertItem(lpI->loadIcon("kvi_join.gif"),"Join channel",this,SLOT(slotActJoin()));
	m_lpActionsPopup->insertItem(lpI->loadIcon("kvi_part.gif"),"Part channel",this,SLOT(slotActPart()));
	m_lpActionsPopup->insertSeparator();
	m_lpActionsPopup->insertItem(lpI->loadIcon("kvii_qery.gif"),"Query user",this,SLOT(slotActQuery()));
	m_lpActionsPopup->insertItem(lpI->loadIcon("kvii_chat.gif"),"DCC Chat",this,SLOT(slotActChat()));
	m_lpActionsPopup->insertItem(lpI->loadIcon("kvii_send.gif"),"DCC Send",this,SLOT(slotActSend()));
	m_lpActionsPopup->insertSeparator();
	m_lpActionsPopup->insertItem(lpI->loadIcon("kvi_away.gif"),"Set AWAY",this,SLOT(slotActAway()));
	m_lpActionsPopup->insertItem(lpI->loadIcon("kvi_back.gif"),"Unset AWAY",this,SLOT(slotActUnaway()));
	m_lpActionsPopup->insertSeparator();
	m_lpActionsPopup->insertItem(lpI->loadIcon("kvi_chmod.gif"),"Set user mode",m_lpUModePopup);
	
	m_lpMenuBar->insertItem("&Main",m_lpMainPopup,1);
	m_lpMenuBar->insertItem("&Settings",m_lpSettingsPopup,2);
	m_lpMenuBar->insertItem("&Actions",m_lpActionsPopup,3);
	m_lpMenuBar->insertItem(m_lpPopupManager->szActionPopup.data(),m_lpPopupManager->m_lpUserActionPopup,4);
	m_lpMenuBar->insertItem("&Window",m_lpMdi->m_lpWinListPopup,5);
	m_lpMenuBar->insertItem("&Help",_macro_kviApplication->getHelpMenu(false,KVI_HELP_ABOUT_STRING),6);

	setMenu(m_lpMenuBar);

	m_lpMenuBar->setItemEnabled(3,false);
	m_lpMenuBar->setMenuBarPos(KMenuBar::Top);
	m_lpMenuBar->setStyle(MotifStyle); //No focus grabbing,thanx...
	m_lpMenuBar->setFocusPolicy(QWidget::NoFocus);
	m_lpMenuBar->setFocusProxy(m_lpMdi);
	QObjectList *lpL=m_lpMenuBar->queryList("QWidget");
	if(lpL){
		QObjectListIt it(*lpL);
		QObject *obj=0;
		while( (obj=it.current()) != 0){
			++it;
			((QWidget *)obj)->setStyle(MotifStyle);
			// The above command is the ONLY way that I found to avoid ANNOYING
			// focus grabbing by the QMenuBar...
			((QWidget *)obj)->setFocusPolicy(NoFocus);
			((QWidget *)obj)->setFocusProxy(m_lpMdi);
		}
	} else _debug_warning("queryList returns NULL!");
	delete lpL;
	_debug_leavetrace("createMenuBar");
}
//============ saveScript ============//
void KviFrame::saveScript()
{
	_debug_entertrace("saveScript");
	KviScriptSaveDialog theDlg(this);
	m_bFocusManagingEnabled=false;
	theDlg.exec();
	m_bFocusManagingEnabled=true;
	_debug_leavetrace("saveScript");
}
//============ loadScript ============//
void KviFrame::loadScript()
{
	_debug_entertrace("loadScript");
	QString szHome=getKVIrcHomeDirectory("Scripts");
	QString szFileName=KFileDialog::getOpenFileName(szHome.data(),0,this,0);
	if(szFileName.isEmpty())return;
	m_bFocusManagingEnabled=false;
	KviScriptDescription d;
	if(!m_lpScriptManager->preloadHeaderAndCheckFile(szFileName.data(),&d)){
		KMsgBox::message(this,"Script loading error","The file is corrupted or unreadable.\nCan't load the script.");
		m_bFocusManagingEnabled=true;
		return;
	}
	KviScriptLoadDialog theDlg(this,&d,szFileName.data());
	if(!theDlg.exec()){
		m_bFocusManagingEnabled=true;
		return;
	}
	saveAllOptions();
	m_bRestarting=true;
	KMsgBox::message(this,"Script succesfully loaded","The settings need to be re-readed.\nKVirc will restart now.");
	m_bFocusManagingEnabled=true;
	_macro_kviApplication->m_bRestart=true;
	_macro_kviApplication->quit();
	_debug_leavetrace("loadScript");
}
//============ saveAllOptions ============//
void KviFrame::saveAllOptions()
{
	_debug_entertrace("saveAllOptions");
	m_lpAliasManager->saveToFile();
	m_lpEventManager->saveToFile();
	m_lpPopupManager->saveToFile();
	m_lpInt->saveSettings();
	m_lpClr->saveSettings();
	m_lpUsr->saveSettings();
	m_lpSrv->saveSettings();
	m_lpOpt->saveSettings();
	_debug_leavetrace("saveAllOptions");
}
//============ newWindow ============//
void KviFrame::newWindow()
{
	_debug_entertrace("newWindow");
	//By now just run a new KVIrc application executable
	saveAllOptions();
	KProcess *proc=new KProcess();
	*proc << "kvirc";
	proc->start(KProcess::DontCare,KProcess::NoCommunication);
	delete proc;
	_debug_leavetrace("newWindow");
}
//============ slotUModeI ============//
void KviFrame::slotUModeI()
{ if(m_lpSock->m_bConnected)m_lpSock->sendFmtData("MODE %s +i",m_lpGlb->szNick.data());}
//============ slotUModeW ============//
void KviFrame::slotUModeW()
{ if(m_lpSock->m_bConnected)m_lpSock->sendFmtData("MODE %s +w",m_lpGlb->szNick.data());}
//============ slotUModeS ============//
void KviFrame::slotUModeS()
{ if(m_lpSock->m_bConnected)m_lpSock->sendFmtData("MODE %s +s",m_lpGlb->szNick.data());}
//============ slotActJoin ============//
void KviFrame::slotActJoin()
{
	_debug_entertrace("slotActJoin");
	QString szToDo="/SET $i.n.t $%box_input[Enter the channel to join];if ($i.n.t) JOIN $i.n.t;UNSET $i.n.t";
	m_lpUserParser->parseUserInput(m_lpConsole,KVI_WND_TYPE_STATUS,KVI_STATUS_WND_NAME,szToDo);
	_debug_leavetrace("slotActJoin");
}
//============ slotActQuery ============//
void KviFrame::slotActQuery()
{
	_debug_entertrace("slotActQuery");
	QString szToDo="/SET $i.n.t $%box_input[Enter the nickname to query];if ($i.n.t) QUERY $i.n.t;UNSET $i.n.t";
	m_lpUserParser->parseUserInput(m_lpConsole,KVI_WND_TYPE_STATUS,KVI_STATUS_WND_NAME,szToDo);
	_debug_leavetrace("slotActQuery");
}
//============ slotActChat ============//
void KviFrame::slotActChat()
{
	_debug_entertrace("slotActChat");
	QString szToDo="/SET $i.n.t $%box_input[Enter the nickname to request DCC Chat];if ($i.n.t) CHAT $i.n.t;UNSET $i.n.t";
	m_lpUserParser->parseUserInput(m_lpConsole,KVI_WND_TYPE_STATUS,KVI_STATUS_WND_NAME,szToDo);
	_debug_leavetrace("slotActChat");
}
//============ slotActAway ============//
void KviFrame::slotActAway()
{
	_debug_entertrace("slotActAway");
	QString szToDo="/AWAY";
	m_lpUserParser->parseUserInput(m_lpConsole,KVI_WND_TYPE_STATUS,KVI_STATUS_WND_NAME,szToDo);
	_debug_leavetrace("slotActAway");
}
//============ slotActUnaway ============//
void KviFrame::slotActUnaway()
{
	_debug_entertrace("slotActUnaway");
	QString szToDo="/BACK";
	m_lpUserParser->parseUserInput(m_lpConsole,KVI_WND_TYPE_STATUS,KVI_STATUS_WND_NAME,szToDo);
	_debug_leavetrace("slotActUnaway");
}
//============ slotActSend ============//
void KviFrame::slotActSend()
{
	_debug_entertrace("slotActSend");
	QString szToDo="/SET $i.n.t $%box_input[Enter the nickname to request DCC Send];if ($i.n.t) SEND $i.n.t;UNSET $i.n.t";
	m_lpUserParser->parseUserInput(m_lpConsole,KVI_WND_TYPE_STATUS,KVI_STATUS_WND_NAME,szToDo);
	_debug_leavetrace("slotActSend");
}
//============ slotActPart ============//
void KviFrame::slotActPart()
{
	_debug_entertrace("slotActPart");
	QString szToDo="/SET $i.n.t $%box_input[Enter the channel to part];";
	szToDo+="if ($i.n.t)< set $i.n.d $%box_input[Enter the part message];";
	szToDo+="if ($i.n.d) part $i.n.t $i.n.d;>; unset $i.n.t ; unset $i.n.d";
	m_lpUserParser->parseUserInput(m_lpConsole,KVI_WND_TYPE_STATUS,KVI_STATUS_WND_NAME,szToDo);
	_debug_leavetrace("slotActPart");
}
//============ setFocusTargetRecursive ============//
void KviFrame::setFocusTargetRecursive(QWidget *lpW,QWidget *lpTrg)
{
	_debug_entertrace("setFocusTargetRecursive");
	if(lpW != lpTrg){
		QList<QObject> *list = (QList<QObject> *)(lpW->children());
		if(list){
			for (unsigned int i=0; i< list->count(); i++){
				QObject *w = list->at(i);
				if(w->inherits("QWidget")){
					setFocusTargetRecursive(((QWidget *)w),lpTrg);
				}
			}
		}
		if(lpW != this)lpW->setFocusPolicy(QWidget::NoFocus);
		lpW->setFocusProxy(lpTrg);
	}
	_debug_leavetrace("setFocusTargetRecursive");
}
//============ dnsQueryDone ============//
void KviFrame::dnsQueryDone(const char *szResult)
{
	_debug_entertrace("dnsQueryDone");
	KviDnsResultStruct dr;
	KviMdiChild *lpC=m_lpMdi->topChildInZOrder();
	if(!lpC)lpC=m_lpConsole;
	if(m_lpDns->getResult(szResult,&dr)){
		lpC->doFmtOutput(KVI_OUT_DNS,"[DNS] : query  : %s",dr.szQuery.data());
		lpC->doFmtOutput(KVI_OUT_DNS,"[DNS] : host   : %s",dr.szHostName.data());
		lpC->doFmtOutput(KVI_OUT_DNS,"[DNS] : ip     : %s",dr.szIp.data());
		if((dr.szAlias1.length()>6)||(strncasecmp(dr.szAlias1.data(),"none",4)))
			lpC->doFmtOutput(KVI_OUT_DNS,"[DNS] : alias1 : %s",dr.szAlias1.data());
		if((dr.szAlias2.length()>6)||(strncasecmp(dr.szAlias2.data(),"none",4)))
			lpC->doFmtOutput(KVI_OUT_DNS,"[DNS] : alias2 : %s",dr.szAlias2.data());
	} else {
		lpC->doFmtOutput(KVI_OUT_DNS,"[DNS] : query  : %s",dr.szQuery.data());
		lpC->doFmtOutput(KVI_OUT_DNS,"[DNS] : error  : %s",dr.szError.data());
	}
	_debug_leavetrace("dnsQueryDone");
}
//============ createStatusBar ============//
void KviFrame::createStatusBar()
{
	_debug_entertrace("createStatusBar");
	m_lpStatusBar = new KviStatusBar(this,"KviStatusBarClass");
	m_lpStatusBar->setLabelText("Ready...");
	addToolBar(m_lpStatusBar);
	m_lpStatusBar->setBarPos(KToolBar::Bottom);
	_debug_leavetrace("createStatusBar");
}
//============ queryExternalDestroy ============//
void KviFrame::queryExternalDestroy(KviMdiChild *lpC)
{
	_debug_entertrace("queryExternalDestroy");
	if(m_lpDestroyTimer->isActive())destroySlot();
	m_lpLastChildToDestroy=lpC;
	m_lpDestroyTimer->start(KVI_DESTROY_TIMEOUT);
	_debug_leavetrace("queryExternalDestroy");
}
//============ destroySlot ============//
void KviFrame::destroySlot()
{
	_debug_entertrace("destroySlot");
	m_lpDestroyTimer->stop();
	m_lpLastChildToDestroy->killWindow();
	m_lpLastChildToDestroy=0;
	_debug_leavetrace("destroySlot");
}
//============ setStatus ============//
void KviFrame::setStatusText(const char *szText)
{
	_debug_entertrace("setStatusText");
	m_lpStatusBar->setLabelText(szText);
	_debug_leavetrace("setStatusText");
}
//============ createToolBar ============//
void KviFrame::createToolBar()
{
	_debug_entertrace("createToolBar");
	m_lpToolBar = new KToolBar(this,"KviToolBar");
	KIconLoader *lpI=_macro_kviIconLoader;

	QFrame *lp1=new QFrame(m_lpToolBar); // Separators....Let's believe that Qt
	QFrame *lp2=new QFrame(m_lpToolBar); // will delete it for us... :)
	QFrame *lp3=new QFrame(m_lpToolBar); //
	QFrame *lp4=new QFrame(m_lpToolBar); //
	QFrame *lp5=new QFrame(m_lpToolBar);

	m_lpToolBar->insertButton(lpI->loadIcon("kvii_conn.gif"),1,SIGNAL(clicked()),this,SLOT(connectionSlot()),true,"Connect");
	m_lpToolBar->insertButton(lpI->loadIcon("kvii_link.gif"),2,SIGNAL(clicked()),this,SLOT(doLinksDialog()),true,"Query links");
	m_lpToolBar->insertWidget(1000,2,lp1);
	m_lpToolBar->insertButton(lpI->loadIcon("kvii_usr.gif"),3,SIGNAL(clicked()),this,SLOT(doUserDialog()),true,"User settings...");
	m_lpToolBar->insertButton(lpI->loadIcon("kvii_srv.gif"),4,SIGNAL(clicked()),this,SLOT(doServerDialog()),true,"Servers...");
	m_lpToolBar->insertButton(lpI->loadIcon("kvii_opt.gif"),5,SIGNAL(clicked()),this,SLOT(doOptionsDialog()),true,"Misc options...");
	m_lpToolBar->insertWidget(1001,2,lp2);
	m_lpToolBar->insertButton(lpI->loadIcon("kvii_int.gif"),6,SIGNAL(clicked()),
								this,SLOT(doInterfaceDialog()),true,"User interface settings...");
 	m_lpToolBar->insertButton(lpI->loadIcon("kvii_clr.gif"),7,SIGNAL(clicked()),
								this,SLOT(doIrcColorsDialog()),true,"Irc colors settings..."); 	
	m_lpToolBar->insertWidget(1002,2,lp3);
	m_lpToolBar->insertButton(lpI->loadIcon("kvii_alias.gif"),8,SIGNAL(clicked()),
								this,SLOT(doAliasDialog()),true,"Aliases...(alias Macros:) )");
 	m_lpToolBar->insertButton(lpI->loadIcon("kvii_event.gif"),9,SIGNAL(clicked()),
								this,SLOT(doEventDialog()),true,"Events");
	m_lpToolBar->insertButton(lpI->loadIcon("kvii_pop.gif"),10,SIGNAL(clicked()),
								this,SLOT(doPopupDialog()),true,"Popups");
	m_lpToolBar->insertWidget(1003,2,lp4);
	m_lpToolBar->insertButton(lpI->loadIcon("kvii_load.gif"),11,SIGNAL(clicked()),
								this,SLOT(loadScript()),true,"Load a script");
	m_lpToolBar->insertButton(lpI->loadIcon("kvii_save.gif"),12,SIGNAL(clicked()),this,SLOT(saveScript()),true,"Save current script");
	m_lpToolBar->insertWidget(1004,2,lp5);
	m_lpToolBar->insertButton(lpI->loadIcon("kvi_help.gif"),13,SIGNAL(clicked()),_macro_kviApplication,SLOT(appHelpActivated()),true,"Help index");
 	m_lpToolBar->insertButton(lpI->loadIcon("kvi_kvirc.gif"),14,SIGNAL(clicked()),this,SLOT(slotAboutKvirc()),true,"Version");


	lp1->setFrameStyle(QFrame::Sunken|QFrame::Panel);
	lp2->setFrameStyle(QFrame::Sunken|QFrame::Panel);
	lp3->setFrameStyle(QFrame::Sunken|QFrame::Panel);
	lp4->setFrameStyle(QFrame::Sunken|QFrame::Panel);
	lp5->setFrameStyle(QFrame::Sunken|QFrame::Panel);

	connect(m_lpToolBar,SIGNAL(moved(BarPosition)),this,SLOT(toolBarMoved()));
	addToolBar(m_lpToolBar);
	m_lpToolBar->setFocusProxy(m_lpMdi);
	_debug_leavetrace("createToolBar");
}
//============ slotAboutKvirc ============//
void KviFrame::slotAboutKvirc()
{
	_debug_entertrace("slotAboutKvirc");
	KviAboutDialog *pDlg=new KviAboutDialog(this);
	m_bFocusManagingEnabled=false;
	pDlg->exec();
	m_bFocusManagingEnabled=true;
	delete pDlg;
	_debug_leavetrace("slotAboutKvirc");
}
//============ toolBarMoved ============//
void KviFrame::toolBarMoved()
{
	_debug_entertrace("toolBarMoved");
	KToolBar::BarPosition pos=m_lpToolBar->barPos();
	KToolBarButton *lpBtn=m_lpToolBar->getButton(1);
	int wdth = 26;
	int hght = 26;
	if(lpBtn){
		wdth=lpBtn->width();
		hght=lpBtn->height();
	}
	if((pos==KToolBar::Left) || (pos==KToolBar::Right))hght=2; else wdth=2;
	QWidget *lpTF=m_lpToolBar->getWidget(1000);
	lpTF->resize(wdth,hght);
	lpTF=m_lpToolBar->getWidget(1001);
	lpTF->resize(wdth,hght);
	lpTF=m_lpToolBar->getWidget(1002);
	lpTF->resize(wdth,hght);
	lpTF=m_lpToolBar->getWidget(1003);
	lpTF->resize(wdth,hght);
	lpTF=m_lpToolBar->getWidget(1004);
	lpTF->resize(wdth,hght);
	_debug_leavetrace("toolBarMoved");
}
//============ doPopupDialog ============//
void KviFrame::doPopupDialog()
{
	_debug_entertrace("doPopupDialog");
	KviPopupDialog *pDlg=new KviPopupDialog(this,m_lpPopupManager);
	m_bFocusManagingEnabled=false;
	pDlg->exec();
	m_bFocusManagingEnabled=true;
	delete pDlg;
	m_lpMenuBar->changeItem(m_lpPopupManager->szActionPopup.data(),4);
	m_lpChanPopup->changeItem(m_lpPopupManager->szChanPopup.data(),7);
	m_lpNickListPopup->changeItem(m_lpPopupManager->szNickListPopup.data(),7);
//	m_lpMenuBar->repaint();
	_debug_leavetrace("doPopupDialog");
}
//============ doServerDialog ============//
void KviFrame::doServerDialog()
{
	_debug_entertrace("doServerDialog");
	KviSrvDialog *pDlg=new KviSrvDialog(this,m_lpSrv);
	m_bFocusManagingEnabled=false;
	pDlg->exec();
	m_bFocusManagingEnabled=true;
	delete pDlg;
	_debug_leavetrace("doServerDialog");
}
//============ doAliasDialog ============//
void KviFrame::doAliasDialog()
{
	_debug_entertrace("doAliasDialog");
	KviAliasDialog *pDlg=new KviAliasDialog(this);
	m_bFocusManagingEnabled=false;
	pDlg->exec();
	m_bFocusManagingEnabled=true;
	delete pDlg;
	_debug_leavetrace("doAliasDialog");
}
//============ slot doSettingsDialog ============//
void KviFrame::doInterfaceDialog()
{
	_debug_entertrace("doInterfaceDialog");
	KviInterfaceDialog *pDlg=new KviInterfaceDialog(this,m_lpInt);
	m_bFocusManagingEnabled=false;
	pDlg->exec();
	m_bFocusManagingEnabled=true;
	delete pDlg;
	applyOptions();
	_debug_leavetrace("doInterfaceDialog");
}
//============ doEventDialog ============//
void KviFrame::doEventDialog()
{
	_debug_entertrace("doEventDialog");
	KviEventDialog *pDlg=new KviEventDialog(this);
	m_bFocusManagingEnabled=false;
	pDlg->exec();
	m_bFocusManagingEnabled=true;
	delete pDlg;
	_debug_leavetrace("doEventDialog");
}
//============ doOptionsDialog ============//
void KviFrame::doOptionsDialog()
{
	_debug_entertrace("doOptionsDialog");
	KviOptDialog *pDlg=new KviOptDialog(this,m_lpOpt);
	m_bFocusManagingEnabled=false;
	pDlg->exec();
	m_bFocusManagingEnabled=true;
	delete pDlg;
	setCaption(m_lpOpt->szCaption.data());
	applyOptions();
	if(m_bLoggedIn && m_lpSock->m_bConnected)startNotifyList();
	_debug_leavetrace("doOptionsDialog");
}
//============ startNotifyList ============//
void KviFrame::startNotifyList()
{
	_debug_entertrace("startNotifyList");
	m_szNotifyString="";
	const char *szNk=0;
	while(!m_lpIsOnList->isEmpty())m_lpIsOnList->removeLast();
	m_lpConsole->m_lpNotifyList->removeAll();
	for(szNk=m_lpOpt->lpNotifyList->first();szNk;szNk=m_lpOpt->lpNotifyList->next()){
		if(!m_szNotifyString.isEmpty())m_szNotifyString+=' ';
		int *lpInt=new int;
		*lpInt=0;
		m_lpIsOnList->append(lpInt);
		m_szNotifyString+=szNk;
	}
	if(m_szNotifyString.length()>500)m_szNotifyString.resize(501);
	if(m_lpNotifyTimer->isActive())m_lpNotifyTimer->stop();
	if((!m_lpSock->m_bConnected) ||(!m_bLoggedIn) 
				|| (!m_lpOpt->bEnableNotifyList) || (m_szNotifyString.isEmpty()))return;
	checkNotifyList();
	m_lpNotifyTimer->start(m_lpOpt->iNotifyTimer * 1000);
	_debug_leavetrace("startNotifyList");
}
//============ checkNotifyList ============//
void KviFrame::checkNotifyList()
{
	_debug_entertrace("checkNotifyList");
	if((!m_lpSock->m_bConnected) || (!m_lpOpt->bEnableNotifyList) || (m_szNotifyString.isEmpty())){
		if(m_lpNotifyTimer->isActive())m_lpNotifyTimer->stop();
		m_lpConsole->m_lpNotifyList->removeAll();
		return;
	}
	m_lpSock->sendFmtData("ISON %s",m_szNotifyString.data());
	_debug_leavetrace("checkNotifyList");
}
//============ slot doUserDialog ============//
void KviFrame::doUserDialog()
{
	_debug_entertrace("doUserDialog");
	KviUserDialog *pDlg=new KviUserDialog(this,m_lpUsr);
	m_bFocusManagingEnabled=false;
	pDlg->exec();
	m_bFocusManagingEnabled=true;
	delete pDlg;
	_debug_leavetrace("doUserDialog");
}
//============ doIrcColorsDialog ============//
void KviFrame::doIrcColorsDialog()
{
	_debug_entertrace("doIrcColorsDialog");
	KviMircColorsDialog *pDlg=new KviMircColorsDialog(this,m_lpClr);
	m_bFocusManagingEnabled=false;
	pDlg->exec();
	m_bFocusManagingEnabled=true;
	delete pDlg;
	_debug_leavetrace("doIrcColorsDialog");
}
//============ focusInEvent ============//
void KviFrame::focusInEvent(QFocusEvent *)
{
	_debug_entertrace("focusInEvent");
	if(!m_bFocusManagingEnabled)return;
	m_lpMdi->focusTopChild();
	_debug_leavetrace("focusInEvent");
}
//============ handleConnectedToServer ============//
void KviFrame::handleConnectedToServer()
{
	_debug_entertrace("handleConnectedToServer");
	m_bLoggedIn=false;
	m_bUsingAltNick=false;
	if(!m_lpLinksDialog)m_lpToolBar->setItemEnabled(2,true);	

	m_lpGlb->szNetworkName	= m_szCurNetwork.copy();
	m_lpGlb->szServerName	= m_hostDescription.szName.copy();
	m_lpGlb->szServerIp		= m_hostDescription.szIp.copy();
	m_lpGlb->szPort.sprintf("%d",m_hostDescription.iPort);
	m_lpGlb->startTime		= time(0);
	m_lpGlb->endTime		= time(0);

	m_lpConsole->doFmtOutput(KVI_OUT_CONNECT,"Connected to %s (%s:%d)",
		m_hostDescription.szName.data(),m_hostDescription.szIp.data(),m_hostDescription.iPort);
	//Get local host here...
	if(m_lpSock->getLocalHostIp(m_lpGlb->szLocalHostIp)==KVI_SOCKERR_SUCCESS){
		m_lpConsole->doFmtOutput(KVI_OUT_INTERNAL,"Localhost %s",m_lpGlb->szLocalHostIp.data());
	} else {
		if(!m_lpOpt->szLocalIp.isEmpty())m_lpGlb->szLocalHostIp="127.0.0.1";
		else m_lpGlb->szLocalHostIp=m_lpOpt->szLocalIp.copy();
		m_lpConsole->doFmtOutput(KVI_OUT_ERROR,"Unable to resolve local host (using %s)",m_lpGlb->szLocalHostIp.data());
	}
	if(m_lpEventManager->lpEvent[KVI_Event_OnConnect]->bEnabled){
			QString szPara=m_lpGlb->szServerName+" "+m_lpGlb->szServerIp+" "+m_lpGlb->szNetworkName+" ";
			szPara+=m_lpGlb->szPort+" "+m_lpGlb->szLocalHostIp;
			m_lpUserParser->executeEvent(m_lpConsole,m_lpEventManager->lpEvent[KVI_Event_OnConnect],szPara);
	}
	if(!(m_szPassword.isEmpty())){
		m_lpConsole->doFmtOutput(KVI_OUT_INTERNAL,"Sending %s as password",m_szPassword.data());
		m_lpSock->sendFmtData("PASS %s",m_szPassword.data());
	}
	if(m_lpUsr->szNick.isEmpty())m_lpUsr->szNick="unknown";
	if(m_lpUsr->szRealName.isEmpty())m_lpUsr->szRealName="unknown";
	if(m_lpUsr->szUserName.isEmpty())m_lpUsr->szUserName="unknown";
	m_lpConsole->doFmtOutput(KVI_OUT_INTERNAL,"Logging in as %s (%s , %s)",m_lpUsr->szNick.data(),
						m_lpUsr->szUserName.data(),m_lpUsr->szRealName.data());
	m_lpGlb->szNick		= m_lpUsr->szNick.copy();
	m_lpGlb->szUserName	= m_lpUsr->szUserName.copy();
	m_lpGlb->szRealName	= m_lpUsr->szRealName.copy();
	m_lpSock->sendFmtData("USER %s %s %s :%s",m_lpUsr->szUserName.data(),m_lpGlb->szLocalHostIp.data(),
				m_lpGlb->szServerName.data(),m_lpUsr->szRealName.data());
	m_lpSock->sendFmtData("NICK %s",m_lpUsr->szNick.data());
	m_lpMenuBar->setItemEnabled(3,true);
	m_lpMenuBar->repaint();
	_debug_leavetrace("handleConnectedToServer");
}
//============ handleProxyReply ============//
void KviFrame::handleProxyReply(unsigned char replycode)
{
	_debug_entertrace("handleProxyReply");
	QString reply;
	switch(replycode){
		case 90:
			reply="Request granted";
			break;
		case 91:
			reply="Request rejected";
			break;
		case 92:
			reply="Request rejected : Unable to connect to your ident server";
			break;
		case 93:
			reply="Request rejected : Ident mismatch";
			break;
	}
	m_lpConsole->doFmtOutput(KVI_OUT_INTERNAL,"[ SOCKS4 ] : Received proxy response %u [%s]",
		(uint)replycode,reply.data());
	if(replycode==90)handleConnectedToServer();
	m_bUsingProxy=false;
	_debug_leavetrace("handleProxyReply");
}
//============ handleSocketConnected ============//
void KviFrame::handleSocketConnected()
{
	_debug_entertrace("handleSocketConnected");
	if(m_bUsingProxy){
		QString szStat="Connected to "+m_proxyDescription.szName;
		setStatusText(szStat.data());
		m_lpConsole->doFmtOutput(KVI_OUT_CONNECT,"[ SOCKS4 ] : Connected to %s (%s:%d)",
			m_proxyDescription.szName.data(),m_proxyDescription.szIp.data(),m_proxyDescription.iPort);
		m_lpConsole->doOutput(KVI_OUT_INTERNAL,"[ SOCKS4 ] : Negotiating proxy connection.");
		//write data to proxy
		QString userAndPass=m_lpSrv->szProxyUsername+" "+m_lpSrv->szProxyPassword;
		int iLen=userAndPass.length()+9;
		char *bufToSend=new char[iLen];
		bzero(bufToSend,iLen);
		bufToSend[0]=(unsigned char)4;
		bufToSend[1]=(unsigned char)1;
		Q_UINT16 port=htons(m_hostDescription.iPort);
		memmove(bufToSend+2,(char *)&port,2);
		Q_UINT32 host=(Q_UINT32)m_hostDescription.iAddress.s_addr;
		memmove(bufToSend+4,(char *)&host,4);
		memmove(bufToSend+8,userAndPass.data(),userAndPass.length());
		m_lpSock->writeRawData(bufToSend,iLen);
		delete[] bufToSend;
		//now wait for the response
	} else {
		QString szStat="Connected to "+m_hostDescription.szName;
		setStatusText(szStat.data());
		handleConnectedToServer();
	}
	_debug_leavetrace("handleSocketConnected");
}
//============ handleConnectFailed ============//
void KviFrame::handleConnectFailed()
{
	_debug_entertrace("handleConnectFailed");
	setStatusText("Connect failed");
	m_lpStatusBar->resetLedState();
	m_lpConsole->doFmtOutput(KVI_OUT_ERROR,"Connect failed [ %s (%s:%d) ]",
			m_hostDescription.szName.data(),m_hostDescription.szIp.data(),m_hostDescription.iPort);
	m_lpMainPopup->changeItem(_macro_kviIconLoader->loadIcon("kvii_conn.gif"),"Connect",0);
	m_lpToolBar->setButtonPixmap(1,_macro_kviIconLoader->loadIcon("kvii_conn.gif"));
	m_bUsingProxy=false;
	if(m_bCanReconnect){
		if(m_lpSrv->reconnectOnConnectFailed){
			m_iCurHostIdx++;
			if((uint)m_iCurHostIdx<m_lpSrv->serverList.count()){
				m_lpConsole->doOutput(KVI_OUT_INTERNAL,"New connection attempt in 5 secs. (Hit 'abort' to stop)");
				m_lpMainPopup->changeItem(_macro_kviIconLoader->loadIcon("kvii_disc.gif"),"Abort",0);	
				m_lpToolBar->setButtonPixmap(1,_macro_kviIconLoader->loadIcon("kvii_disc.gif"));	
				m_bReconnecting=true;
				m_lpReconnectTimer->start(5000);
			} else m_lpConsole->doOutput(KVI_OUT_ERROR,"Can't hop server: No more servers available");
		}
	}
	_debug_leavetrace("handleConnectFailed");
}
//============ handleSocketDisconnected ============//
void KviFrame::handleSocketDisconnected()
{
	_debug_entertrace("handleSocketDisconnected");
	m_lpStatusBar->resetLedState();
	setStatusText("Disconnected");

	if(m_lpUserHostTimer->isActive())m_lpUserHostTimer->stop();
	if(m_lpNotifyTimer->isActive())m_lpNotifyTimer->stop();
	m_lpConsole->m_lpNotifyList->removeAll();

	m_lpMainPopup->changeItem(_macro_kviIconLoader->loadIcon("kvii_conn.gif"),"Connect",0);
	m_lpToolBar->setButtonPixmap(1,_macro_kviIconLoader->loadIcon("kvii_conn.gif"));
	m_lpMenuBar->setItemEnabled(3,false);
	m_lpToolBar->setItemEnabled(2,false);
	m_lpMenuBar->repaint();

	m_bUsingProxy=false;

	KviMdiChild *lpC=m_lpMdi->m_lpChildList->first();
	QList<KviMdiChild> list;
	for(lpC=m_lpMdi->m_lpChildList->first();lpC;lpC=m_lpMdi->m_lpChildList->next()){
		if((lpC->type()==KVI_WND_TYPE_CHAN) ||(lpC->type()==KVI_WND_TYPE_QUERY)){
			list.append(lpC);
		}
	}
	list.setAutoDelete(false);
	while(!list.isEmpty()){
		lpC=list.first();
		list.removeRef(lpC);
		m_lpMdi->removeWindow(lpC);
	}
	m_lpConsole->setFocus();
	m_lpConsole->doFmtOutput(KVI_OUT_DISCONNECT,"Connection terminated [ %s (%s:%d) ]",
			m_hostDescription.szName.data(),m_hostDescription.szIp.data(),m_hostDescription.iPort);

	if(m_lpEventManager->lpEvent[KVI_Event_OnDisconnect]->bEnabled){
			QString szPara=m_hostDescription.szName+" "+m_hostDescription.szIp+" ";
			m_lpUserParser->executeEvent(m_lpConsole,m_lpEventManager->lpEvent[KVI_Event_OnDisconnect],szPara);
	}
	if(m_bCanReconnect){
		if(m_lpSrv->reconnectOnDisconnect){
			m_lpConsole->doOutput(KVI_OUT_INTERNAL,"New connection attempt in 5 secs (Hit 'abort' to stop)");
			m_lpMainPopup->changeItem(_macro_kviIconLoader->loadIcon("kvii_disc.gif"),"Abort",0);	
			m_lpToolBar->setButtonPixmap(1,_macro_kviIconLoader->loadIcon("kvii_disc.gif"));
			m_bReconnecting=true;
			m_lpReconnectTimer->start(5000);
		}
	}
	_debug_leavetrace("handleSocketDisconnected");
}
//============ handleSocketError ============//
void KviFrame::handleSocketError(int nErr)
{
	_debug_entertrace("handleSocketError");
	QString szError;
	m_lpSock->getErr(nErr,szError);
	m_lpConsole->doFmtOutput(KVI_OUT_ERROR,"[SOCKET ERROR]:%s",szError.data());
	_debug_leavetrace("handleSocketError");
}
//============ connectToCurrentServer ============//
void KviFrame::connectToCurrentServer()
{
	_debug_entertrace("connectToCurrentServer");
	//Setup
	if(m_lpUsr->szNick.isEmpty() || m_lpUsr->szUserName.isEmpty() || m_lpSrv->serverList.isEmpty()){
		m_lpConsole->doOutput(KVI_OUT_ERROR,"Can't connect: Please select settings/User from the menu bar");
		return;
	}

	m_bUsingProxy=false;
	//Get the server to connect to
	QString szSrv(m_lpSrv->serverList.at(m_iCurHostIdx));
	KviServer kS(szSrv);

	m_lpConsole->doFmtOutput(KVI_OUT_INTERNAL,"Attempting connection to %s (%s) on port %d",kS.szServerName.data(),
								kS.szNetworkName.data(),kS.iPort);


	if(m_lpSrv->bUseProxy){ //attempt to use the proxy
		m_lpConsole->doOutput(KVI_OUT_INTERNAL,"[ SOCKS4 ] : Preparing proxy informations.");
		m_proxyDescription.szName   = m_lpSrv->szProxyAddress.copy();
		if(m_proxyDescription.szName.isEmpty()){
			m_lpConsole->doOutput(KVI_OUT_ERROR,"[ SOCKS4 ] : Invalid proxy address : Resuming direct connection.");
			m_lpSrv->bUseProxy=false;
		} else {
			m_proxyDescription.szIp     = m_lpSrv->szProxyIp.copy();
			m_proxyDescription.szAlias1 = "unknown";
			m_proxyDescription.szAlias2 = "unknown";
			bool bOk=false;
			m_proxyDescription.iPort    = m_lpSrv->szProxyPort.toUShort(&bOk);
			if(!bOk){
				m_lpConsole->doOutput(KVI_OUT_ERROR,"[ SOCKS4 ] : Invalid proxy port : Resuming direct connection.");
				m_lpSrv->bUseProxy=false;
			} else { //ok ...look for the host
				QString szStat="Looking up host "+m_proxyDescription.szName;
				setStatusText(szStat.data());
				if(inet_aton(m_proxyDescription.szIp.data(),&(m_proxyDescription.iAddress))){
					m_lpConsole->doFmtOutput(KVI_OUT_INTERNAL,"[ SOCKS4 ] : Proxy name resolved (cached address) : %s",
										m_proxyDescription.szIp.data()); 					
					m_bUsingProxy=true;
				} else {
					m_lpSrv->szProxyIp="";
					m_lpConsole->doFmtOutput(KVI_OUT_INTERNAL,"[ SOCKS4 ] : Looking up host %s",
										m_proxyDescription.szName.data());
					int nErr=m_lpSock->getHostByName(m_proxyDescription.szName.data(),&m_proxyDescription);
					if(nErr != KVI_SOCKERR_SUCCESS){
						m_lpConsole->doFmtOutput(KVI_OUT_ERROR,
							"[ SOCKS4 ] : Unable to resolve proxy name : Resuming direct connection.");
 						m_lpSrv->bUseProxy=false;
					} else {
						m_lpSrv->szProxyIp=m_proxyDescription.szIp.copy();
						m_lpConsole->doFmtOutput(KVI_OUT_INTERNAL,"[ SOCKS4 ] : Proxy name resolved : %s",
									m_proxyDescription.szIp.data());
						m_bUsingProxy=true;
					}
				}
			}
		}
	}

	m_bCanReconnect=true;
	m_bReconnecting=false;
	clearHostDescription();

	m_szCurNetwork=kS.szNetworkName.data();
	m_szPassword=(kS.szPassword.isEmpty() ? "" : kS.szPassword.data());
	QString szStat="Looking up host "+kS.szServerName;
	setStatusText(szStat.data());
	_macro_kviApplication->processEvents();
	bool bDone=false;
	if(m_lpSrv->cacheLookup && (!kS.szIp.isEmpty()) && (kS.szIp != "Resolve at connect")){
		if(inet_aton(kS.szIp.data(),&(m_hostDescription.iAddress))){
			bDone=true;
			m_lpConsole->doOutput(KVI_OUT_INTERNAL,"Server hostname resolved (address found in cache).");
			m_hostDescription.szName	=kS.szServerName.data();
			m_hostDescription.szAlias1	="unknown";
			m_hostDescription.szAlias2	="unknown";
			m_hostDescription.szIp		=kS.szIp.data();
			m_hostDescription.iPort		=kS.iPort;
		} 
	} 
    if(!bDone){
		int nErr=m_lpSock->getHostByName(kS.szServerName.data(),&m_hostDescription);
		if(nErr != KVI_SOCKERR_SUCCESS){
			handleSocketError(nErr);
			m_hostDescription.szName	=kS.szServerName.data();
			m_hostDescription.szIp		=kS.szIp.data();
			m_hostDescription.iPort		=kS.iPort;
			handleConnectFailed();
			clearHostDescription();
			return;
		} else {
			if(m_lpSrv->cacheLookup){
				kS.szIp	=m_hostDescription.szIp.data();
				m_lpSrv->serverList.remove(m_iCurHostIdx); //not the first!!!
				QString szS;
				kS.toString(szS);
				m_lpSrv->serverList.insert(m_iCurHostIdx,szS.data());
				m_lpConsole->doOutput(KVI_OUT_INTERNAL,"Server hostname resolved (lookup cached)");
			} else {
				m_lpConsole->doOutput(KVI_OUT_INTERNAL,"Server hostname resolved");
			}
			m_hostDescription.iPort=kS.iPort;
		}
	}
	connectToServer();
	_debug_leavetrace("connectToCurrentServer");
}
//============ clearHostDescription ============//
void KviFrame::clearHostDescription()
{
	_debug_entertrace("clearHostDescription");
	m_hostDescription.szName	="none";
	m_hostDescription.szAlias1	="none";
	m_hostDescription.szAlias2	="none";
	m_hostDescription.szIp		="none";
	m_hostDescription.iPort		=0;
	_debug_leavetrace("clearHostDescription");
}
//============ connectTo ============//
void KviFrame::connectToServer()
{
	_debug_entertrace("connectToServer");
	m_bLoggedIn=false;
	m_lpMainPopup->changeItem(_macro_kviIconLoader->loadIcon("kvii_disc.gif"),"Disconnect",0);
	m_lpToolBar->setButtonPixmap(1,_macro_kviIconLoader->loadIcon("kvii_disc.gif"));
	int nErr;
	if(m_bUsingProxy){
		QString szStat="Connecting to "+m_proxyDescription.szName+" ("+m_proxyDescription.szIp+":";
		QString szPort;
		szPort.setNum(m_proxyDescription.iPort);
		szStat+=szPort+")";
		setStatusText(szStat.data());
		nErr=m_lpSock->connectTo(m_proxyDescription.iAddress,m_proxyDescription.iPort);
	} else {
		QString szStat="Connecting to "+m_hostDescription.szName+" ("+m_hostDescription.szIp+":";
		QString szPort;
		szPort.setNum(m_hostDescription.iPort);
		szStat+=szPort+")";
		setStatusText(szStat.data());
		nErr=m_lpSock->connectTo(m_hostDescription.iAddress,m_hostDescription.iPort);
	}
	if(nErr != KVI_SOCKERR_SUCCESS){
		handleSocketError(nErr);
		handleConnectFailed();
		return;
	}
	_debug_leavetrace("connectToServer");
}
//============ disconnectFromServer ============//
void KviFrame::disconnectFromServer()
{
	_debug_entertrace("disconnectFromServer");
	if( ! (m_lpSock->m_bInProgress || m_lpSock->m_bConnected))return;
	if(m_lpSock->m_bConnected){
		QString szQuit=( m_lpOpt->szQuitMsg.isEmpty() ? "Leaving" : m_lpOpt->szQuitMsg.data() );
		m_lpUserParser->m_lpIdentifiers->substitute(m_lpConsole,szQuit,0);
		m_lpSock->sendFmtData("QUIT :%s",szQuit.data());
	}
	m_bCanReconnect=false;
	m_iCurHostIdx=0;
	m_lpSock->disconnect();
	_debug_leavetrace("disconnectFromServer");
}
//============ reconnectionSlot ============//
void KviFrame::reconnectionSlot()
{
	_debug_entertrace("reconnectionSlot");
	m_bReconnecting=false;
	m_lpReconnectTimer->stop();
	m_lpMainPopup->changeItem(_macro_kviIconLoader->loadIcon("kvii_conn.gif"),"Connect",0);
	m_lpToolBar->setButtonPixmap(1,_macro_kviIconLoader->loadIcon("kvii_conn.gif"));
	connectToCurrentServer();
	_debug_leavetrace("reconnectionSlot");
}
//============ hopServer ============//
void KviFrame::hopServer(QString &szServer,int iPort)
{
	_debug_entertrace("hopServer");
	if(szServer.isEmpty())return;
	KviServer newSrv;
	newSrv.iPort=iPort;
	newSrv.szServerName=szServer.copy();
	QString szS;
	newSrv.toString(szS);
	const char *pS=0;
	bool bFnd=false;
	//look for a server entry with the same server name
	for(pS=m_lpSrv->serverList.first();pS && (!bFnd);pS=m_lpSrv->serverList.next()){
		QString szStr=pS;
		KviServer srv(szStr);
		if(!strcasecmp(srv.szServerName.data(),szServer.data())){
			//found somewhere in the list
			//move it to the first location
			bFnd=true;
			srv.iPort = iPort; //use the specified port
			QString szSrv;
			srv.toString(szSrv);
			m_lpSrv->serverList.removeRef(pS);
			m_lpSrv->serverList.insert(0,szSrv.data());
		}
	}
	if(!bFnd){ //Not found...add it
		m_lpSrv->serverList.insert(0,szS.data());
	}

	if(m_lpSock->m_bConnected || m_bReconnecting || m_lpSock->m_bInProgress){
		//allow returning the function before destroying windows....
		QTimer::singleShot(200,this,SLOT(hopServerSlot()));
		return;
	}
	m_iCurHostIdx=0;
	connectToCurrentServer();
	_debug_leavetrace("hopServer");
}
//============ hopServerSlot ============//
void KviFrame::hopServerSlot()
{
	_debug_entertrace("hopServerSlot");
	if(m_lpSock->m_bConnected || m_bReconnecting || m_lpSock->m_bInProgress){
		connectionSlot();
	}
	_macro_kviApplication->processEvents();
	m_iCurHostIdx=0;
	connectToCurrentServer();
	_debug_leavetrace("hopServerSlot");
}
//============ connectionSlot ============//
// Connected to the 'Connect' menu item
void KviFrame::connectionSlot()
{
	_debug_entertrace("connectionSlot");
	if(m_bReconnecting){ //Stop reconnecting
		if(m_lpReconnectTimer->isActive())m_lpReconnectTimer->stop();
		m_bReconnecting=false;
		m_bCanReconnect=false;
		m_lpMainPopup->changeItem(_macro_kviIconLoader->loadIcon("kvii_conn.gif"),"Connect",0);
		m_lpToolBar->setButtonPixmap(1,_macro_kviIconLoader->loadIcon("kvii_conn.gif"));
		m_lpConsole->doOutput(KVI_OUT_INTERNAL,"Connection aborted");
		return;
	}
	if(m_lpSock->m_bConnected || m_lpSock->m_bInProgress){ //Disconnect
		disconnectFromServer();
	} else { //Connect
		m_iCurHostIdx=0;
		connectToCurrentServer();
	}
	_debug_leavetrace("connectionSlot");
}
//============ requestNickListPopup ============//
void KviFrame::requestNickListPopup(QPoint pnt)
{
	_debug_entertrace("requestNickListPopup");
	if(m_lpGlb->lpCurrentChan && (!m_lpGlb->szCurrentSelection.isEmpty())){
		pnt.setX(pnt.x()-80);
		m_lpNickListPopup->popup(pnt);
	}
	_debug_leavetrace("requestNickListPopup");
}
//============ requestNotifyListPopup ============//
void KviFrame::requestNotifyListPopup(QPoint pnt)
{
	_debug_entertrace("requestNotifyListPopup");
	if(!m_lpGlb->szCurrentNotifySelection.isEmpty()){
		pnt.setX(pnt.x()-80);
		m_lpNotifyListPopup->popup(pnt);
	}
	_debug_leavetrace("requestNotifyListPopup");
}
//============ queryUserHost ============//
void KviFrame::queryUserHost(QString &szUserHost)
{
	_debug_entertrace("queryUserHost");
	if(m_szNextUserHostQuery.isEmpty()){
		m_szNextUserHostQuery=szUserHost.copy();
		if(!m_lpUserHostTimer->isActive()){
			m_lpUserHostTimer->start(m_lpOpt->iUserListUpdateTime * 1000);
			doUserHost();
		}
	}
	_debug_leavetrace("queryUserHost");
}
//============ doUserHost ============//
void KviFrame::doUserHost()
{
	_debug_entertrace("doUserHost");
	if(m_szNextUserHostQuery.isEmpty()||(!m_lpSock->m_bConnected)){
		if(m_lpUserHostTimer->isActive())m_lpUserHostTimer->stop();
	} else {
		m_lpSock->sendFmtData("USERHOST %s",m_szNextUserHostQuery.data());
		m_szNextUserHostQuery="";
	}
	_debug_leavetrace("doUserHost");
}
#include "m_kvi_frame.moc"
