//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//

#ifndef _KVI_FRAME_H_INCLUDED_
#define _KVI_FRAME_H_INCLUDED_

typedef struct pixmapTabTag{
	int			index;
	const char 	*szPixName;	
	const char	*szDescription;
} pixmapTable;

class KviFrame : public KTopLevelWidget
{
	Q_OBJECT;
public:
	static pixmapTable pixTab[KVI_NUM_PIXMAPS];
public:
	KviFrame();
	~KviFrame();
public:    //variables
	KviInterface       *m_lpInt;
	KviMircColors      *m_lpClr;
	KviUser            *m_lpUsr;
	KviSrv             *m_lpSrv;
	KviOptions         *m_lpOpt;
	QColor             *m_lpColor[16]; //managed by m_lpClr!!! created and deleted!
	QPixmap            *m_lpPixmap[KVI_NUM_PIXMAPS];
	KviStatusWnd       *m_lpConsole;
	KviIrcSocket       *m_lpSock;
	KviStatusBar       *m_lpStatusBar;
	KviServerParser    *m_lpServerParser;
	KviGlobals         *m_lpGlb;
	KviMdiManager      *m_lpMdi;
	KviUserParser      *m_lpUserParser;
	KviAliasManager    *m_lpAliasManager;
	KviEventManager    *m_lpEventManager;
	KviPopupManager    *m_lpPopupManager;
	KviTranslator      *m_lpTranslator;
	KviScriptManager   *m_lpScriptManager;
	KviLinksDialog     *m_lpLinksDialog;
	KviSetup           *m_lpSetup;
	QList<int>         *m_lpIsOnList;
	KviAudio           *m_lpAudio;
	KviNetscapeController *m_lpNetscape;
	KviDns             *m_lpDns;
	KviAsciiPlayer     *m_lpAsciiPlayer;
	bool               m_bUsingProxy;
	bool               m_bLoggedIn;
	bool               m_bUsingAltNick;
	bool               m_bCanReconnect;
	bool               m_bReconnecting;
	bool               m_bFocusManagingEnabled;
	bool               m_bRestarting;
	kviHostDescription m_hostDescription;
	kviHostDescription m_proxyDescription;
	int                m_iCurHostIdx;
public:    //methods
	void linksDialogClosed();
	void formatNickOutputString(QString &szResult,QString &szNick);
	void setStatusText(const char *szText);
	void applyOptions();
	void handleSocketConnected();
	void handleConnectFailed();
	void handleSocketDisconnected();
	void handleProxyReply(unsigned char replycode);
	void handleConnectedToServer();
	void handleSocketError(int nErr);
	void queryExternalDestroy(KviMdiChild *lpC);
	void connectToCurrentServer();
	void connectToServer();
	void hopServer(QString &szServer,int iPort);
	void disconnectFromServer();
	void clearHostDescription();
	void requestNickListPopup(QPoint pnt);
	void requestNotifyListPopup(QPoint pnt);
	void requestChanPopup(QPoint pnt);
	void requestStatusPopup(QPoint pnt);
	void requestChatPopup(QPoint pnt);
	void viewDoubleClicked(QString &szString);
	void requestQueryPopup(QPoint pnt);
	void setFocusTargetRecursive(QWidget *lpW,QWidget *lpTrg);
	void queryUserHost(QString &szUserHost);
	void startNotifyList();
	KviChanWnd  * createChanWnd(const char *szName);
	KviQueryWnd * createQueryWnd(const char *szName);
	KviChatWnd  * createChatWnd(const char *szName);
	KviSendWnd  * createSendWnd(const char *szName,bool bMinimized);
	KviUserWnd  * createUserWnd(const char *szName,bool bMinimized,bool bHasInput,const char *szImage=0,QRect *geom=0);
private:    //variables
	KMenuBar        *m_lpMenuBar;
	KToolBar        *m_lpToolBar;
	QPopupMenu      *m_lpSettingsPopup;
	QPopupMenu      *m_lpMainPopup;
	QPopupMenu      *m_lpNickListPopup;
	QPopupMenu      *m_lpNotifyListPopup;
	QPopupMenu      *m_lpControlPopup;
	QPopupMenu      *m_lpDCCPopup;
	QPopupMenu      *m_lpCTCPPopup;
	QPopupMenu      *m_lpInfoPopup;
	QPopupMenu		*m_lpChanPopup;
	QPopupMenu		*m_lpToolPopup;
	QPopupMenu      *m_lpStatusPopup;
	QPopupMenu      *m_lpStatsPopup;
	QPopupMenu      *m_lpActionsPopup;
	QPopupMenu      *m_lpUModePopup;
	QString         m_szCurNetwork;
	QString         m_szPassword;
	QTimer          *m_lpReconnectTimer;
	QRect           m_statusWndRect;
	bool            m_bStatusWndMaximized;
	QTimer          *m_lpDestroyTimer;
	QTimer          *m_lpUserHostTimer;
	QTimer          *m_lpNotifyTimer;
	QString         m_szNextUserHostQuery;
	QString			m_szNotifyString;
	int             m_iNextChildIdx;
	KviMdiChild     *m_lpLastChildToDestroy;
private:    //methods
	void readSettings();
	void saveSettings();
	void createMenuBar();
	void createStatusBar();
	void createToolBar();
	void createMdiManager();
	void createColors();
	void destroyColors();
	void createPixmaps();
	void destroyPixmaps();
	void createNickListPopup();
	void createNotifyListPopup();
	void createChanPopup();
	void createStatusPopup();
private slots:
	void doLinksDialog();
	void doEventDialog();
	void doUserDialog();
	void doInterfaceDialog();
	void doOptionsDialog();
	void doIrcColorsDialog();
	void doServerDialog();
	void doPopupDialog();
	void doAliasDialog();
	void doUserHost();
	void connectionSlot();
	void reconnectionSlot();
	void destroySlot();
	void checkNotifyList();
	void hopServerSlot();
	void newWindow();
	void saveAllOptions();
	void saveScript();
	void loadScript();
	void dnsQueryDone(const char *szResult);

	void nickListUserSlot(const char *szCommand);
	void nickListSlot(const char *szCommand);
	void chanUserSlot(const char *szCommand);
	void chatUserSlot(const char *szCommand);
	void queryUserSlot(const char *szCommand);

	void slotUserAction(const char *szAction);

	void slotOp();
	void slotDeop();
	void slotVoice();
	void slotDevoice();
	void slotQuery();
	void slotPing();
	void slotChat();
	void slotVersion();
	void slotTime();
	void slotClientinfo();
	void slotUserinfo();
	void slotFinger();
	void slotWhois();
	void slotKick();
	void slotDCCSend();
	void slotKBan();
	void slotWhowas();
	void slotWhoWho();
	void slotUserlist();
	void slotClonescan();

	void notifyListSlot(const char *szCommand);

	void slotNotifyWhois();
	void slotNotifyPing();
	void slotNotifyDCCChat();
	void slotNotifyDCCSend();
	void slotNotifyVersion();
	void slotNotifyQuery();

	void slotChanKvirc();
	void slotChanUserlist();
	void slotChanClonescan();
	void slotChanBanlist();
	void slotChanPart();

	void slotStatusMotd();
	void slotStatusVersion();
	void slotStatusTime();
	void slotStatusAdmin();
	void slotStatusInfo();
	void slotStatusUsers();
	void slotStatusLusers();
//	void slotStatusLinks();
	void slotStatusStatsC();
	void slotStatusStatsK();
	void slotStatusStatsU();
	void slotStatusStatsM();

	void statusSlot(const char *sendData);

	void slotUModeI();
	void slotUModeS();
	void slotUModeW();

	void slotActJoin();
	void slotActPart();
	void slotActQuery();
	void slotActChat();
	void slotActSend();
	void slotActAway();
	void slotActUnaway();
	void slotAboutKvirc();

	void toolBarMoved();
protected:
	virtual void focusInEvent(QFocusEvent *);
};

#endif // _KVI_FRAME_H_INCLUDED_
