//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//

//#define _KVI_DEBUG_CLASS_NAME_ "KviListBox"

#include "kvi_app.h"

KviListBox::KviListBox(QWidget *parent,KviMdiChild *lpParent,KviFrame *lpFrame,const char *name):QListBox(parent,name)
{
	_debug_entertrace("KviListBox");
	m_lpParent=lpParent;
	m_lpMainFrame=lpFrame;
	m_lpInt=lpFrame->m_lpInt;
	setTableFlags(Tbl_autoVScrollBar | Tbl_snapToGrid);
	setBackgroundMode(QWidget::NoBackground);
	setMultiSelection(true);
	m_lpNickList=new QList<KviNick>;
	m_lpNickList->setAutoDelete(true);
	m_lpMemBuffer=new QPixmap(width(),height());
	m_lpfM=new QFontMetrics(font());
	_debug_leavetrace("KviListBox");
}
KviListBox::~KviListBox(){
	while(!m_lpNickList->isEmpty())m_lpNickList->removeLast();
	delete m_lpNickList;
	delete m_lpMemBuffer;
	delete m_lpfM;
}

//============ removeAll ============//

void KviListBox::removeAll()
{
	_debug_entertrace("removeAll");
	clear();
	while(!m_lpNickList->isEmpty())m_lpNickList->removeLast();
	repaint();
	_debug_leavetrace("removeAll");
}


void KviListBox::paintCell(QPainter *p,int row,int col){
	const char *sztext=text( row );
	if ( isSelected( row ) ) {
		p->fillRect( 0, 0, cellWidth(col), cellHeight(row),m_lpInt->clr_bk_sel_listbox); //select back
		p->setPen(m_lpInt->clr_fr_sel_listbox); //select fore
    } else {
		if(!sztext)return;
		QString szText(sztext);
		if(szText.at(0)=='@')p->setPen(m_lpInt->clr_fr_op_listbox); //op fore
		else if(szText.at(0)=='+')p->setPen(m_lpInt->clr_fr_voice_listbox); //voice fore
		else p->setPen(m_lpInt->clr_fr_normal_listbox); //fore
    }
	if(!sztext)return;
    p->drawText( 3,  m_lpfM->ascent() + m_lpfM->leading()/2, sztext );
}
void KviListBox::paintEvent(QPaintEvent *){

	uint cnt=count();

	QRect updateR = frameRect();			// update rectangle
	updateScrollBars();

	QPainter paint;

	paint.begin(cnt ? m_lpMemBuffer : this);
	
	paint.setFont(font());

	QRect internalR=contentsRect();
	QBrush bkBrush(m_lpInt->clr_bk_listbox); //back
	if((!(m_lpInt->pix_bk_listbox.isNull())))bkBrush.setPixmap(m_lpInt->pix_bk_listbox);

	if(!cnt)internalR=rect();

	paint.fillRect(internalR,bkBrush);
	drawFrame(&paint);

	if(!cnt){
		paint.end();
		return;
	}

    int row = findRow( internalR.y() );
    int col = findCol( internalR.x() );
   	int xStart, yStart;

    if ( !colXPos( col, &xStart ) || !rowYPos( row, &yStart ) ) {
		paint.end();
		return;
    }
    int	  maxY	= internalR.bottom();
    int	  nextY;
    QWMatrix matrix;
	paint.setClipRect(internalR);
	(*m_lpfM)=paint.fontMetrics();
    while ( yStart <= maxY && row < numRows() ) {
		nextY = yStart + cellHeight( row );
		matrix.translate( xStart, yStart );
		paint.setWorldMatrix( matrix);
		paintCell( &paint, row, col );
		matrix.reset();
		paint.setWorldMatrix( matrix );
		row++;
		yStart = nextY;
    }
	paint.end();
	bitBlt(this,updateR.top(),updateR.left(),m_lpMemBuffer,0,0,-1,-1,CopyROP,FALSE);
}

//============ resizeEvent ============//

void KviListBox::resizeEvent(QResizeEvent *e)
{
	_debug_entertrace("resizeEvent");
	m_lpMemBuffer->resize(width(),height());
	QListBox::resizeEvent(e);
	_debug_leavetrace("resizeEvent");
}


////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
// NICK
////////////////////////
//
//	JOIN
//
void KviListBox::nickJoin(KviNick &kvNick){
	if(findNick(kvNick) != -1){
		debug("%s is already in the list box...mmh",kvNick.szNick.data());
		return;
	} else {
		insertNick(kvNick);
	}
}
void KviListBox::nickJoin(QString &szNick){
	KviNick kN(szNick);
	nickJoin(kN);
}
void KviListBox::nickJoin(const char *szNick){
	QString szNk=szNick;
	nickJoin(szNk);
}
//
//	PART
//
void KviListBox::nickPart(KviNick &kvNick){
	int idx=findNick(kvNick);
	if(idx == -1){
		debug("%s is not in the list box.",kvNick.szNick.data());
	} else {
		m_lpNickList->remove(idx);
		removeItem(idx);
	}
}
void KviListBox::nickPart(QString &szNick){
	KviNick kN(szNick);
	nickPart(kN);
}
void KviListBox::nickPart(const char *szNick){
	QString szNk=szNick;
	nickPart(szNk);
}
//
//	FIND
//
int KviListBox::findNick(KviNick &kvNick){
	KviNick *lpN=0;
	int idx=0;
	for(lpN=m_lpNickList->first();lpN;lpN=m_lpNickList->next()){
		if(lpN->matches(kvNick.szNick))return idx;
		idx++;
	}
	return -1; //Not found
}
int KviListBox::findNick(QString &szNick){
	KviNick *lpN=0;
	int idx=0;
	for(lpN=m_lpNickList->first();lpN;lpN=m_lpNickList->next()){
		if(lpN->matches(szNick))return idx;
		idx++;
	}
	return -1; //Not found
}
int KviListBox::findNick(const char *szNick){
	QString szNk=szNick;
	return findNick(szNk);
}
//
//	INSERTNICK
//
void KviListBox::insertNick(KviNick &kvNick){
	QString szNiko=kvNick.szNick.copy();
	if(kvNick.bOp){
		szNiko.prepend("@");
	}
	if(kvNick.bVoice){
		szNiko.prepend("+");
	}
	if(m_lpNickList->isEmpty()){
		m_lpNickList->append(new KviNick(kvNick));
		insertItem(szNiko.data());
		return;
	} else {
		uint cnt=m_lpNickList->count();
		for(uint i=0;i<cnt;i++){
			const char *szN=text(i);
			const char *szNew=szNiko.data();
			if(*szNew=='+'){
				if(*szN!='@'){ //compare and insert
					if(strcasecmp(szN,szNew)>=0){ //szN is greater
						insertItem(szNew,i);
						m_lpNickList->insert(i,new KviNick(kvNick));
						return;
					}	
				} //else skip...+ goes after
			} else if(*szNew=='@'){
				if(*szN=='+'){ //Voiced one...insert at this position
					insertItem(szNew,i);
					m_lpNickList->insert(i,new KviNick(kvNick));
					return;
				} else {
					if(strcasecmp(szN,szNew)>=0){ //szN is greater
						insertItem(szNew,i);
						m_lpNickList->insert(i,new KviNick(kvNick));
						return;
					}
				}
			} else if(strcasecmp(szN,szNew)>=0){ //szN is greater
				insertItem(szNew,i);
				m_lpNickList->insert(i,new KviNick(kvNick));
				return;
			}
		}
		insertItem(szNiko.data());
		m_lpNickList->append(new KviNick(kvNick));
	}
}
void KviListBox::mousePressEvent(QMouseEvent *e){
	if(e->button() & RightButton){
		if(m_lpParent->type()==KVI_WND_TYPE_CHAN){
			m_lpMainFrame->m_lpGlb->lpCurrentChan=(KviChanWnd *)m_lpParent;
			m_lpMainFrame->m_lpGlb->szCurrentSelection="";
			uint cnt=count();
			for(uint i=0;i<cnt;i++){
				if(isSelected(i)){
					if(!m_lpMainFrame->m_lpGlb->szCurrentSelection.isEmpty()){
						m_lpMainFrame->m_lpGlb->szCurrentSelection+=",";
					}
					QString szTxt=text(i);
					while((szTxt[0]=='@')||(szTxt[0]=='+'))szTxt.remove(0,1);
					m_lpMainFrame->m_lpGlb->szCurrentSelection+=szTxt;
				}
			}
			if(!m_lpMainFrame->m_lpGlb->szCurrentSelection.isEmpty()){
				m_lpMainFrame->requestNickListPopup(mapToGlobal(e->pos()));
			}
		} else { //Status window
			m_lpMainFrame->m_lpGlb->szCurrentNotifySelection="";
			uint cnt=count();
			for(uint i=0;i<cnt;i++){
				if(isSelected(i)){
					if(!m_lpMainFrame->m_lpGlb->szCurrentNotifySelection.isEmpty()){
						m_lpMainFrame->m_lpGlb->szCurrentNotifySelection+=",";
					}
					QString szTxt=text(i);
					while((szTxt[0]=='@')||(szTxt[0]=='+'))szTxt.remove(0,1);
					m_lpMainFrame->m_lpGlb->szCurrentNotifySelection+=szTxt;
				}
			}
			if(!m_lpMainFrame->m_lpGlb->szCurrentNotifySelection.isEmpty()){
				m_lpMainFrame->requestNotifyListPopup(mapToGlobal(e->pos()));
			}	
		}
	} else {
		if(!((e->state() & ShiftButton)||(e->state() & ControlButton)))clearSelection();
		QListBox::mousePressEvent(e);
	}
}
//============ mouseDoubleClickEvent ============//
void KviListBox::mouseDoubleClickEvent(QMouseEvent *e)
{
	_debug_entertrace("mouseDoubleClickEvent");
	QListBox::mouseDoubleClickEvent(e);
	int index=currentItem();
	if(m_lpParent->type()==KVI_WND_TYPE_CHAN){
		if(index >= 0){
			KviNick *lpN=m_lpNickList->at(index);
			if(lpN){
				QString szIdent;
				szIdent=lpN->szNick.copy();
				szIdent+=" is ";
				szIdent+=lpN->szNick;
				szIdent+='!';
				int nlen=strlen(KVI_STR_NULL);
				if(strncmp(lpN->szUserName.data(),KVI_STR_NULL,nlen)){
					szIdent+=lpN->szUserName;
				} else szIdent+='*';
				szIdent+='@';
				if(strncmp(lpN->szAddress.data(),KVI_STR_NULL,nlen)){
					szIdent+=lpN->szAddress;
				} else szIdent+='*';
				m_lpMainFrame->setStatusText(szIdent.data());
			}
		}
	}
	_debug_leavetrace("mouseDoubleClickEvent");
}

#include "m_kvi_listbox.moc"
