//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CLASS_NAME_ "KviOptDialog"

#include "kvi_app.h"
#include "kvi_translate.h"
#include <kfiledialog.h>

KviOptDialog::KviOptDialog(KviFrame *lpParent,KviOptions *lpOpt):QTabDialog(lpParent,"Options dialog",true)
{
	m_lpOpt=lpOpt;
	m_lpFrm=lpParent;

	resize(420,250);

	setCaption("Misc options");

	//+++++++++++++++++++
	//	Messages tab
	//+++++++++++++++++++

	lpMsgTab=new QWidget(this);

	lpCaption   = new QLineEdit(lpMsgTab);
	lpQuit      = new QLineEdit(lpMsgTab);
	lpPart      = new QLineEdit(lpMsgTab);
	lpKick      = new QLineEdit(lpMsgTab);
	lpAway      = new QLineEdit(lpMsgTab);
	lpFinger    = new QLineEdit(lpMsgTab);
	lpUserInfo  = new QLineEdit(lpMsgTab);
	lpScript    = new QLineEdit(lpMsgTab);

	QLabel *lpM1 = new QLabel("Caption text",lpMsgTab);
	QLabel *lpM2 = new QLabel("Quit message",lpMsgTab);
	QLabel *lpM3 = new QLabel("Part message",lpMsgTab);
	QLabel *lpM4 = new QLabel("Kick message",lpMsgTab);
	QLabel *lpM5 = new QLabel("Away message",lpMsgTab);
	QLabel *lpM6 = new QLabel("Finger reply",lpMsgTab);
	QLabel *lpM7 = new QLabel("Userinfo reply",lpMsgTab);
	QLabel *lpM8 = new QLabel("Script version",lpMsgTab);

	lpM1->setGeometry(10,20,95,26);
	lpM2->setGeometry(10,50,95,26);
	lpM3->setGeometry(10,80,95,26);
	lpM4->setGeometry(10,110,95,26);
	lpM5->setGeometry(10,140,95,26);
	lpM6->setGeometry(10,170,95,26);
	lpM7->setGeometry(10,200,95,26);
	lpM8->setGeometry(10,230,95,26);

	lpCaption->setText(m_lpOpt->szCaption.data());
	lpQuit->setText(m_lpOpt->szQuitMsg.data());
	lpPart->setText(m_lpOpt->szPartMsg.data());
	lpKick->setText(m_lpOpt->szKickMsg.data());
	lpAway->setText(m_lpOpt->szAwayMsg.data());
	lpFinger->setText(m_lpOpt->szFinger.data());
	lpUserInfo->setText(m_lpOpt->szUserInfo.data());
	lpScript->setText(m_lpOpt->szScriptVer.data());

	lpCaption->setGeometry(110,20,365,26);
	lpQuit->setGeometry(110,50,365,26);
	lpPart->setGeometry(110,80,365,26);
	lpKick->setGeometry(110,110,365,26);
	lpAway->setGeometry(110,140,365,26);
	lpFinger->setGeometry(110,170,365,26);
	lpUserInfo->setGeometry(110,200,365,26);
	lpScript->setGeometry(110,230,365,26);

	//++++++++++++++++++
	//	DCC Tab
	//++++++++++++++++++

	lpDCCTab=new QWidget(this);

	lpAutoChat    = new KviBoolChanger(this,lpDCCTab,240,20,220,26,"Auto accept DCC Chat",&(m_lpOpt->bAutoAcceptDCCChat));
	lpAutoSend    = new KviBoolChanger(this,lpDCCTab,240,50,220,26,"Auto accept DCC Send",&(m_lpOpt->bAutoAcceptDCCSend));
	lpIgnoreChat  = new KviBoolChanger(this,lpDCCTab,10,20,220,26,"Ignore DCC Chat",&(m_lpOpt->bIgnoreDCCChat));
	lpIgnoreSend  = new KviBoolChanger(this,lpDCCTab,10,50,220,26,"Ignore DCC Send",&(m_lpOpt->bIgnoreDCCSend));
	lpAutoClose   = new KviBoolChanger(this,lpDCCTab,240,80,220,26,"Auto close DCC Send window",&(m_lpOpt->bAutoCloseDCCChatOnSuccess));
	lpAutoMinimize= new KviBoolChanger(this,lpDCCTab,10,80,220,26,"Minimize DCC Send window",&(m_lpOpt->bMinimizeDCCSend));
	lpPacket=new KIntegerLine(lpDCCTab);
	lpTimeout=new KIntegerLine(lpDCCTab);
	QLabel *lp8=new QLabel("Dcc Send packet (bytes)[64-4096]",lpDCCTab);
	QLabel *lp9=new QLabel("Dcc Send/Chat timeout (sec.)[>10]",lpDCCTab);

	lp8->setGeometry(10,110,220,26);
	lp9->setGeometry(10,140,220,26);
	lpPacket->setGeometry(240,110,235,26);
	lpTimeout->setGeometry(240,140,235,26);

	connect(lpIgnoreChat,SIGNAL(toggled(bool)),this,SLOT(chatIgnore(bool)));
	connect(lpIgnoreSend,SIGNAL(toggled(bool)),this,SLOT(sendIgnore(bool)));

	lpAutoChat->setEnabled(!m_lpOpt->bIgnoreDCCChat);
	lpAutoSend->setEnabled(!m_lpOpt->bIgnoreDCCSend);
	lpPacket->setValue(m_lpOpt->iDCCSendPacketSize);
	lpTimeout->setValue(m_lpOpt->iDCCCHATTimeout);

	lpMotd    = new QMultiLineEdit(lpDCCTab);
	if(!m_lpOpt->lpListDccMotd->isEmpty()){
		const char *szS=0;
		for(szS=m_lpOpt->lpListDccMotd->first();szS;szS=m_lpOpt->lpListDccMotd->next()){
			lpMotd->append(szS);
		}
	}

	lpMotd->setGeometry(10,240,465,80);

	lpShowMotdOnInDcc = new KviBoolChanger(this,lpDCCTab,10,210,220,26,"Send MOTD on Incoming DCC Chat",&(m_lpOpt->bShowMotdOnInDcc));
	lpShowMotdOnOutDcc= new KviBoolChanger(this,lpDCCTab,240,210,220,26,"Send MOTD on Outgoing DCC Chat",&(m_lpOpt->bShowMotdOnOutDcc));
	connect(lpShowMotdOnInDcc,SIGNAL(toggled(bool)),this,SLOT(enableMotd(bool)));
	connect(lpShowMotdOnOutDcc,SIGNAL(toggled(bool)),this,SLOT(enableMotd(bool)));

	QLabel *lpMotdLabel=new QLabel("Message of the day for the DCC Chat server (MOTD):",lpDCCTab);
	KSeparator *lpSep1=new KSeparator(KSeparator::HLine,lpDCCTab);
	lpSep1->setGeometry(10,175,465,4);
	lpMotdLabel->setGeometry(10,180,405,26);
	lpMotd->setEnabled(m_lpOpt->bShowMotdOnInDcc || m_lpOpt->bShowMotdOnOutDcc);

	//++++++++++++++++++
	//	Notify list Tab
	//++++++++++++++++++

	lpNotTab=new QWidget(this);

	lpNotifyInActive   = new KviBoolChanger(this,lpNotTab,10,300,400,26,"Notify in active window",&(m_lpOpt->bNotifyListInActive));
	lpNotify           = new KviBoolChanger(this,lpNotTab,10,20,210,26,"Enable notify list update every",&(m_lpOpt->bEnableNotifyList));
	lpNotifyTime       = new KIntegerLine(lpNotTab);
	QLabel *lp14       = new QLabel("seconds",lpNotTab);
	lpBox              = new QListBox(lpNotTab);
	lpEdit             = new QLineEdit(lpNotTab);
	lpRem              = new QPushButton("Remove",lpNotTab);
	lpAdd              = new QPushButton("Add",lpNotTab);
	KSeparator *lpSep2 = new KSeparator(KSeparator::HLine,lpNotTab);

	lpSep2->setGeometry(10,50,465,4);
	lpAdd->setGeometry(10,90,120,26);
	lpRem->setGeometry(10,120,120,26);
	lpEdit->setGeometry(10,60,120,26);
	lpBox->setGeometry(135,60,340,240);
	lp14->setGeometry(410,20,60,26);
	lpNotifyTime->setGeometry(240,20,160,26);

	lpNotifyTime->setValue(m_lpOpt->iNotifyTimer);

	lpNotifyTime->setEnabled(lpNotify->isChecked());
	lpNotifyInActive->setEnabled(lpNotify->isChecked());
	lpBox->setEnabled(lpNotify->isChecked());
	lpEdit->setEnabled(lpNotify->isChecked());
	lpRem->setEnabled(lpNotify->isChecked());
	lpAdd->setEnabled(lpNotify->isChecked());

	const char *szNk=0;
	for(szNk=m_lpOpt->lpNotifyList->first();szNk;szNk=m_lpOpt->lpNotifyList->next()){
		lpBox->insertItem(szNk);
	}

	connect(lpRem,SIGNAL(clicked()),this,SLOT(removeNick()));
	connect(lpAdd,SIGNAL(clicked()),this,SLOT(addNick()));
	connect(lpNotify,SIGNAL(toggled(bool)),lpNotifyTime,SLOT(setEnabled(bool)));
	connect(lpNotify,SIGNAL(toggled(bool)),lpNotifyInActive,SLOT(setEnabled(bool)));
	connect(lpNotify,SIGNAL(toggled(bool)),lpBox,SLOT(setEnabled(bool)));
	connect(lpNotify,SIGNAL(toggled(bool)),lpEdit,SLOT(setEnabled(bool)));
	connect(lpNotify,SIGNAL(toggled(bool)),lpRem,SLOT(setEnabled(bool)));
	connect(lpNotify,SIGNAL(toggled(bool)),lpAdd,SLOT(setEnabled(bool)));

	//++++++++++++++++++++++++++++++++
	// Translation & URL catcher tab
	//++++++++++++++++++++++++++++++++

	lpTrnTab=new QWidget(this);

	// translation

	QString szText="Translation map file : ";
	szText+=m_lpOpt->szTranslationFile.data();

	lpUseTran   =new KviBoolChanger(this,lpTrnTab,10,20,465,26,"Use charset translation (You need to set the correct font too)",
									&(m_lpOpt->bUseTranslation));
	lpTranFile  =new QLabel(szText.data(),lpTrnTab);
	lpChooseFile=new QPushButton("Choose translation file...",lpTrnTab);
	lpNotTranDCC=new KviBoolChanger(this,lpTrnTab,10,110,465,26,"Send DCC Chat outgoing data in local charset",
									&(m_lpOpt->bNotTranslateDCC));

	connect(lpUseTran,SIGNAL(toggled(bool)),lpTranFile,SLOT(setEnabled(bool)));
	connect(lpUseTran,SIGNAL(toggled(bool)),lpChooseFile,SLOT(setEnabled(bool)));
	connect(lpChooseFile,SIGNAL(clicked()),this,SLOT(chooseTranslationFile()));
	connect(lpUseTran,SIGNAL(toggled(bool)),lpNotTranDCC,SLOT(setEnabled(bool)));

	lpChooseFile->setGeometry(10,80,465,26);
	lpTranFile->setGeometry(10,50,465,26);

	lpChooseFile->setEnabled(m_lpOpt->bUseTranslation);
	lpTranFile->setEnabled(m_lpOpt->bUseTranslation);
	lpNotTranDCC->setEnabled(m_lpOpt->bUseTranslation);

	// separator
	
	KSeparator *lpSep3 = new KSeparator(KSeparator::HLine,lpTrnTab);
	lpSep3->setGeometry(10,140,465,4);

	//url catcher

	lpBrowserOnUrl = new KviBoolChanger(this,lpTrnTab,10,150,465,26,"Run browser on url double click",(&m_lpOpt->bRunBrowserOnUrl));
	lpUseNetscape  = new KviBoolChanger(this,lpTrnTab,10,180,465,26,"Use netscape remote control instead of running the program below"
											,(&m_lpOpt->bUseNetscape));
 	lpBrowserCmd   = new QLineEdit(lpTrnTab);
	QLabel *lpBrw1 = new QLabel("Browser command line (use % as file name)",lpTrnTab);
	lpHighlightUrl = new KviBoolChanger(this,lpTrnTab,10,280,400,26,"Highlight urls using the mIrc color (0-15)"
											,(&m_lpOpt->bHighlightUrl));
	lpUrlColor     = new KIntegerLine(lpTrnTab);
	KSeparator *lpSep4 = new KSeparator(KSeparator::HLine,lpTrnTab);

	connect(lpBrowserOnUrl,SIGNAL(toggled(bool)),lpUseNetscape,SLOT(setEnabled(bool)));
	connect(lpBrowserOnUrl,SIGNAL(toggled(bool)),lpBrw1,SLOT(setEnabled(bool)));
	connect(lpBrowserOnUrl,SIGNAL(toggled(bool)),lpBrowserCmd,SLOT(setEnabled(bool)));	
	connect(lpHighlightUrl,SIGNAL(toggled(bool)),lpUrlColor,SLOT(setEnabled(bool)));

	lpUseNetscape->setEnabled(m_lpOpt->bRunBrowserOnUrl);
	lpBrw1->setEnabled(m_lpOpt->bRunBrowserOnUrl);
	lpBrowserCmd->setEnabled(m_lpOpt->bRunBrowserOnUrl);
	lpUrlColor->setEnabled(m_lpOpt->bHighlightUrl);

	lpBrw1->setGeometry(10,210,465,26);
	lpBrowserCmd->setGeometry(10,240,465,26);
	lpUrlColor->setGeometry(420,280,55,26);
	lpSep4->setGeometry(10,270,465,4);

	lpBrowserCmd->setText(m_lpOpt->szBrowserCmdLine.data());
	lpUrlColor->setValue((int)m_lpOpt->urlHighlightForeColor);



	lpMisTab=new QWidget(this);

	lpIp=new QLineEdit(lpMisTab);
	QLabel *lp6=new QLabel("Local IP (for multiip machines)",lpMisTab);
	lp6->setGeometry(10,20,210,26);
	lpIp->setGeometry(225,20,250,26);
	lpIp->setText(m_lpOpt->szLocalIp.data());
	
	lpPrfx=new QLineEdit(lpMisTab);
	QLabel *lp7=new QLabel("Personal command prefix",lpMisTab);
	lp7->setGeometry(10,50,180,26);
	lpPrfx->setGeometry(225,50,250,26);
	QString szPr="";
	szPr+=m_lpOpt->chPersonalCommandPrefix;
	lpPrfx->setText(szPr.data());
	lpPrfx->setMaxLength(1);

	QLabel *lp10=new QLabel("Allow max",lpMisTab);
	lpMaxCTCP   =new KIntegerLine(lpMisTab);
	QLabel *lp11=new QLabel("CTCP's in",lpMisTab);
	lpSecCTCP   =new KIntegerLine(lpMisTab);
	QLabel *lp12=new QLabel("seconds (0=ignore)",lpMisTab);

	lp10->setGeometry(10,80,60,26);
	lpMaxCTCP->setGeometry(75,80,40,26);
	lp11->setGeometry(120,80,85,26);
	lpSecCTCP->setGeometry(225,80,40,26);
	lp12->setGeometry(275,80,110,26);

	lpMaxCTCP->setValue(m_lpOpt->iMaxCTCP);
	lpSecCTCP->setValue(m_lpOpt->iTimerCTCP);

	lpUserList=new KviBoolChanger(this,lpMisTab,10,110,213,26,"Enable user list update every",&(m_lpOpt->bEnableUserListUpdate));
	lpUserTime=new KIntegerLine(lpMisTab);
	lpUserTime->setGeometry(225,110,40,26);
	lpUserTime->setValue(m_lpOpt->iUserListUpdateTime);

	connect(lpUserList,SIGNAL(toggled(bool)),lpUserTime,SLOT(setEnabled(bool)));

	lpUserTime->setEnabled(lpUserList->isChecked());
	QLabel *lp13=new QLabel("seconds",lpMisTab);
	lp13->setGeometry(270,110,110,26);

	KSeparator *lpSep5 = new KSeparator(KSeparator::HLine,lpMisTab);
	lpSep5->setGeometry(10,140,465,4);

	QLabel *lpMass1      = new QLabel("Event_OnMassDeop    fires after",lpMisTab);
	QLabel *lpMass2      = new QLabel("Event_OnMassOp      fires after",lpMisTab);
	QLabel *lpMass3      = new QLabel("Event_OnMassKick    fires after",lpMisTab);
	QLabel *lpMass4      = new QLabel("Event_OnMassBan     fires after",lpMisTab);
	QLabel *lpMass5      = new QLabel("Event_OnPublicFlood fires after",lpMisTab);
	QLabel *lpMass1a     = new QLabel("deop(s) in",lpMisTab);
	QLabel *lpMass2a     = new QLabel("op(s) in",lpMisTab);
	QLabel *lpMass3a     = new QLabel("kick(s) in",lpMisTab);
	QLabel *lpMass4a     = new QLabel("ban(s) in",lpMisTab);
	QLabel *lpMass5a     = new QLabel("line(s) in",lpMisTab);
	QLabel *lpMass1b     = new QLabel("second(s)",lpMisTab);
	QLabel *lpMass2b     = new QLabel("second(s)",lpMisTab);
	QLabel *lpMass3b     = new QLabel("second(s)",lpMisTab);
	QLabel *lpMass4b     = new QLabel("second(s)",lpMisTab);
	QLabel *lpMass5b     = new QLabel("second(s)",lpMisTab);
	lpMassDeopTime       = new KIntegerLine(lpMisTab);
	lpMassOpTime         = new KIntegerLine(lpMisTab);
	lpMassKickTime       = new KIntegerLine(lpMisTab);
	lpMassBanTime        = new KIntegerLine(lpMisTab);
	lpPublicFloodTime    = new KIntegerLine(lpMisTab);
	lpMassDeopDeadline   = new KIntegerLine(lpMisTab);
	lpMassOpDeadline     = new KIntegerLine(lpMisTab);
	lpMassKickDeadline   = new KIntegerLine(lpMisTab);
	lpMassBanDeadline    = new KIntegerLine(lpMisTab);
	lpPublicFloodDeadline= new KIntegerLine(lpMisTab);

	lpMass1->setGeometry(10,150,210,26);
	lpMassDeopDeadline->setGeometry(225,150,40,26);
	lpMass1a->setGeometry(270,150,80,26);
	lpMassDeopTime->setGeometry(355,150,40,26);
	lpMass1b->setGeometry(400,150,65,26);

	lpMass2->setGeometry(10,180,210,26);
	lpMassOpDeadline->setGeometry(225,180,40,26);
	lpMass2a->setGeometry(270,180,80,26);
	lpMassOpTime->setGeometry(355,180,40,26);
	lpMass2b->setGeometry(400,180,65,26);

	lpMass3->setGeometry(10,210,210,26);
	lpMassKickDeadline->setGeometry(225,210,40,26);
	lpMass3a->setGeometry(270,210,80,26);
	lpMassKickTime->setGeometry(355,210,40,26);
	lpMass3b->setGeometry(400,210,65,26);

	lpMass4->setGeometry(10,240,210,26);
	lpMassBanDeadline->setGeometry(225,240,40,26);
	lpMass4a->setGeometry(270,240,80,26);
	lpMassBanTime->setGeometry(355,240,40,26);
	lpMass4b->setGeometry(400,240,65,26);

	lpMass5->setGeometry(10,270,210,26);
	lpPublicFloodDeadline->setGeometry(225,270,40,26);
	lpMass5a->setGeometry(270,270,80,26);
	lpPublicFloodTime->setGeometry(355,270,40,26);
	lpMass5b->setGeometry(400,270,65,26);

	lpMassDeopDeadline->setValue(m_lpOpt->massDeopDeadline);
	lpMassDeopTime->setValue(m_lpOpt->massDeopTime);
	lpMassOpDeadline->setValue(m_lpOpt->massOpDeadline);
	lpMassOpTime->setValue(m_lpOpt->massOpTime);
	lpMassKickDeadline->setValue(m_lpOpt->massKickDeadline);
	lpMassKickTime->setValue(m_lpOpt->massKickTime);
	lpMassBanDeadline->setValue(m_lpOpt->massBanDeadline);
	lpMassBanTime->setValue(m_lpOpt->massBanTime);
	lpPublicFloodDeadline->setValue(m_lpOpt->publicFloodDeadline);
	lpPublicFloodTime->setValue(m_lpOpt->publicFloodTime);

	lpMixTab=new QWidget(this);

	lpClone        =new KviBoolChanger(this,lpMixTab,10,20,400,26,"Enable clone scan on join",&(m_lpOpt->bCloneScanOnJoin));
	lpInverseNick  =new KviBoolChanger(this,lpMixTab,10,50,400,26,"Show inverse colored nick in channel messages",&(m_lpOpt->bInverseNick));
	lpPrefixMsg    =new KviBoolChanger(this,lpMixTab,10,80,400,26,"Prefix channel messages with ':'",&(m_lpOpt->bPrefixMsg));
	lpAutoRejoin   =new KviBoolChanger(this,lpMixTab,10,110,400,26,"Autorejoin channel when kicked",&(m_lpOpt->bAutoRejoinOnKick));
	lpShowColorBox =new KviBoolChanger(this,lpMixTab,10,140,400,26,"Popup color box in the input line on CTRL+K",(&m_lpOpt->bShowColorBox));
	lpTimestamp    =new KviBoolChanger(this,lpMixTab,10,170,400,26,"Timestamp messages on the output box",(&m_lpOpt->bTimestamp));
	lpAlignText    =new KviBoolChanger(this,lpMixTab,10,200,400,26,"Align text after a line break",(&m_lpOpt->bAlignText));
	lpShowPixmaps  =new KviBoolChanger(this,lpMixTab,10,230,400,26,"Show message-images in the output box",(&m_lpOpt->bShowPixmaps));

	addTab(lpMsgTab,"Messages");
	addTab(lpDCCTab,"DCC");
	addTab(lpNotTab,"Notify list");
	addTab(lpTrnTab,"Translation/URL");
	addTab(lpMisTab,"Misc 1");
	addTab(lpMixTab,"Misc 2");

	setCancelButton();

	connect(this,SIGNAL(applyButtonPressed()),this,SLOT(applySlot()));

	QResizeEvent e(size(),size());
	resizeEvent(&e);
}


KviOptDialog::~KviOptDialog()
{
	delete lpMassDeopTime;
	delete lpMassOpTime;
	delete lpMassKickTime;
	delete lpMassBanTime;
	delete lpPublicFloodTime;

	delete lpMassDeopDeadline;
	delete lpMassOpDeadline;
	delete lpMassKickDeadline;
	delete lpMassBanDeadline;
	delete lpPublicFloodDeadline;

	delete lpShowColorBox;
	delete lpTimestamp;
	delete lpAlignText;
	delete lpShowPixmaps;
	delete lpHighlightUrl;
	delete lpUrlColor;
	delete lpCaption;
	delete lpQuit;
	delete lpMotd;
	delete lpShowMotdOnOutDcc;
	delete lpShowMotdOnInDcc;
	delete lpBrowserCmd;
	delete lpBrowserOnUrl;
	delete lpUseNetscape;
	delete lpPart;
	delete lpKick;
	delete lpAway;
	delete lpFinger;
	delete lpUserInfo;
	delete lpScript;
	delete lpIp;
	delete lpAutoRejoin;
	delete lpPrefixMsg;
	delete lpPrfx;
	delete lpPacket;
	delete lpTimeout;
	delete lpAutoClose;
	delete lpAutoChat;
	delete lpAutoSend;
	delete lpAutoMinimize;
	delete lpIgnoreChat;
	delete lpIgnoreSend;
	delete lpChooseFile;	
	delete lpMaxCTCP;
	delete lpSecCTCP;
	delete lpInverseNick;
	delete lpUserList;
	delete lpUserTime;
	delete lpClone;
	delete lpNotify;
	delete lpRem;
	delete lpAdd;
	delete lpBox;
	delete lpEdit;
	delete lpNotifyTime;
	delete lpNotifyInActive;
	delete lpUseTran;
	delete lpTranFile;
	delete lpNotTranDCC;
	delete lpDCCTab;
	delete lpMisTab;
	delete lpMsgTab;
	delete lpNotTab;
//	delete lpUrlTab;
	delete lpMixTab;
	delete lpTrnTab;
}

//============ enableMotd ============//

void KviOptDialog::enableMotd(bool bEnable)
{
	_debug_entertrace("enableMotd");
	lpMotd->setEnabled(lpShowMotdOnInDcc->isChecked() || lpShowMotdOnOutDcc->isChecked());
	_debug_leavetrace("enableMotd");
}



//============ resizeEvent ============//

void KviOptDialog::resizeEvent(QResizeEvent *)
{
	_debug_entertrace("resizeEvent");
	setFixedSize(500,400);
	_debug_leavetrace("resizeEvent");
}


//============ addNick ============//

void KviOptDialog::addNick()
{
	_debug_entertrace("addNick");
	if(lpEdit->text()){
		//check that we have enough space
		uint cnt=lpBox->count();
		int charcnt=0;
		for(uint i=0;i<cnt;i++){
			const char *szText=lpBox->text((int)i);
			if(szText){
				charcnt+=strlen(szText);
				charcnt++;
			}
			if(!strcasecmp(lpEdit->text(),szText)){
				KMsgBox::message(this,"Notify list error","This nick is already in the notify list.\n");
				return;
			}
		}
		charcnt+=strlen(lpEdit->text());
		if(charcnt>490){
			KMsgBox::message(this,"Notify list error","Can not add another nick to the notify list.\n"\
												"The ISON message is limited to 510 characters.\n"\
												"Check the current notify list, are you sure that you\n"\
												"really need all the nicks listed?");
			return;
		}
		if(strlen(lpEdit->text())>0){
			lpBox->insertItem(lpEdit->text());
			lpEdit->setText("");
		}
	}
	_debug_leavetrace("addNick");
}

//============ removeNick ============//

void KviOptDialog::removeNick()
{
	_debug_entertrace("removeNick");
	if(lpBox->currentItem() != -1){
		lpBox->removeItem(lpBox->currentItem());
	}
	_debug_leavetrace("removeNick");
}



//============ chatIgnore ============//

void KviOptDialog::chatIgnore(bool bIgnore)
{
	_debug_entertrace("chatIgnore");
	lpAutoChat->setEnabled(!bIgnore);
	_debug_leavetrace("chatIgnore");
}

//============ sendIgnore ============//

void KviOptDialog::sendIgnore(bool bIgnore)
{
	_debug_entertrace("sendIgnore");
	lpAutoSend->setEnabled(!bIgnore);
	_debug_leavetrace("sendIgnore");
}


//============ chooseTranslationFile ============//

void KviOptDialog::chooseTranslationFile()
{
	_debug_entertrace("chooseTranslationFile");
	QString szHome(getKVIrcHomeDirectory("Translation"));
	QString szCmd=QString(KFileDialog::getOpenFileName(szHome.data(),0,m_lpFrm,0));
	if(!szCmd.isEmpty()){
		if(m_lpFrm->m_lpTranslator->setTranslationMap(szCmd.data())){
			m_lpOpt->szTranslationFile=szCmd.data();
			QString szText="Translation map file : ";
			szText+=m_lpOpt->szTranslationFile.data();
			lpTranFile->setText(szText.data());
			m_lpOpt->bUseTranslation=true;
		} else {
			lpUseTran->setChecked(false);
			m_lpOpt->bUseTranslation=false;
			m_lpOpt->szTranslationFile="[__none__]";
			QString szText="Translation map file : ";
			szText+=m_lpOpt->szTranslationFile.data();
			lpTranFile->setText(szText.data());
			lpTranFile->setEnabled(false);
			lpChooseFile->setEnabled(false);
		}
	}

	_debug_leavetrace("chooseTranslationFile");
}



void KviOptDialog::applySlot()
{
	_debug_entertrace("applySlot");
	m_lpOpt->szCaption=(lpCaption->text() ? lpCaption->text() : "");
	m_lpOpt->szQuitMsg=(lpQuit->text()    ? lpQuit->text()    : "");
	m_lpOpt->szPartMsg=(lpPart->text()    ? lpPart->text()    : "");
	m_lpOpt->szKickMsg=(lpKick->text()    ? lpKick->text()    : "");
	m_lpOpt->szFinger =(lpFinger->text()  ? lpFinger->text()  : "");
	m_lpOpt->szAwayMsg=(lpAway->text()    ? lpAway->text()    : "");
	m_lpOpt->szUserInfo=(lpUserInfo->text() ? lpUserInfo->text() : "");
	m_lpOpt->szScriptVer=(lpScript->text() ? lpScript->text() : "");
	m_lpOpt->szLocalIp=(lpIp->text()      ? lpIp->text()      : "127.0.0.1");
	m_lpOpt->szBrowserCmdLine=(lpBrowserCmd->text() ? lpBrowserCmd->text() : "kfmclient openUrl '%'");
	QString szPr=(lpPrfx->text()          ? lpPrfx->text()    : "@");
	if(!szPr.isEmpty())m_lpOpt->chPersonalCommandPrefix=szPr[0];
	m_lpOpt->iDCCCHATTimeout=lpTimeout->value();
	if(m_lpOpt->iDCCCHATTimeout<10)m_lpOpt->iDCCCHATTimeout=10;
	m_lpOpt->iDCCSendPacketSize=lpPacket->value();
	if(m_lpOpt->iDCCSendPacketSize<64)m_lpOpt->iDCCSendPacketSize=64;
	if(m_lpOpt->iDCCSendPacketSize>4096)m_lpOpt->iDCCSendPacketSize=4096;

	m_lpOpt->iTimerCTCP=lpSecCTCP->value();
	if(m_lpOpt->iTimerCTCP<0)m_lpOpt->iTimerCTCP=0;
	m_lpOpt->iMaxCTCP=lpMaxCTCP->value();
	if(m_lpOpt->iMaxCTCP<1)m_lpOpt->iMaxCTCP=1;

	m_lpOpt->iUserListUpdateTime=lpUserTime->value();
	if(m_lpOpt->iUserListUpdateTime<2)m_lpOpt->iUserListUpdateTime=2;
	m_lpOpt->iNotifyTimer=lpNotifyTime->value();
	if(m_lpOpt->iNotifyTimer<5)m_lpOpt->iNotifyTimer=5;

	m_lpOpt->urlHighlightForeColor=(char)lpUrlColor->value();
	if((m_lpOpt->urlHighlightForeColor<0)||(m_lpOpt->urlHighlightForeColor>15))m_lpOpt->urlHighlightForeColor=12;

	m_lpOpt->massDeopDeadline=(Q_UINT32)lpMassDeopDeadline->value();
	m_lpOpt->massDeopTime=(Q_UINT32)lpMassDeopTime->value();
	m_lpOpt->massOpDeadline=(Q_UINT32)lpMassOpDeadline->value();
	m_lpOpt->massOpTime=(Q_UINT32)lpMassOpTime->value();
	m_lpOpt->massKickDeadline=(Q_UINT32)lpMassKickDeadline->value();
	m_lpOpt->massKickTime=(Q_UINT32)lpMassKickTime->value();
	m_lpOpt->massBanDeadline=(Q_UINT32)lpMassBanDeadline->value();
	m_lpOpt->massBanTime=(Q_UINT32)lpMassBanTime->value();
	m_lpOpt->publicFloodDeadline=(Q_UINT32)lpPublicFloodDeadline->value();
	m_lpOpt->publicFloodTime=(Q_UINT32)lpPublicFloodTime->value();

	uint cnt=lpBox->count();
	while(!m_lpOpt->lpNotifyList->isEmpty())m_lpOpt->lpNotifyList->removeLast();
	if(cnt){
		for(uint i=0;i<cnt;i++){
			m_lpOpt->lpNotifyList->append(lpBox->text(i));
		}
	}
	if(m_lpOpt->bUseTranslation){
		if(m_lpOpt->szTranslationFile != "[__none__]"){
			if(!m_lpFrm->m_lpTranslator->setTranslationMap(m_lpOpt->szTranslationFile.data())){
				m_lpOpt->szTranslationFile="[__none__]";
				m_lpOpt->bUseTranslation=false;
			}
		} else m_lpOpt->bUseTranslation=false;
	}
	while(!m_lpOpt->lpListDccMotd->isEmpty())m_lpOpt->lpListDccMotd->removeLast();
	int nLines=lpMotd->numLines();
	if(nLines>0){
		for(int i=0;i<nLines;i++){
			const char *szTextLine=lpMotd->textLine(i);
			if(szTextLine){
				QString szLine=szTextLine;
				if(!szLine.isEmpty()){
					int idx=szLine.find('\n');
					if(idx == -1){
						m_lpOpt->lpListDccMotd->append(szLine.data());
					} else {
						szLine.remove(idx,1);
						if(!szLine.isEmpty())m_lpOpt->lpListDccMotd->append(szLine.data());
					}
				}
			}
		}
	}
	accept();
	
	_debug_leavetrace("applySlot");
}

#include "m_kvi_optdlg.moc"
