//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//
#include "kvi_app.h"

#include <kfiledialog.h>

//
//   Server edit dialog
//

KviServerEditDialog::KviServerEditDialog(KviSrvDialog *parent,KviServer *lpS) : QDialog(parent,"KviEditServerDialog",true)
{
	m_lpS=lpS;
	setCaption("Edit server properties..");
	QString szRestrict="zxcvbnmasdfghjklpoiuytrewqQWERTYUIOPASDFGHJKLZXCVBNM_1234567890.-_";
	lpNetName		= new KRestrictedLine(this,"Net",szRestrict.data());
	lpSrvName		= new KRestrictedLine(this,"Srv",szRestrict.data());
	szRestrict+="\\\"\'|!$?%&/()=^~`}][{@#+*駰.:,;<>";
	lpPassword		= new KRestrictedLine(this,"Pas",szRestrict.data());
	szRestrict+=' ';
	lpDescription   = new KRestrictedLine(this,"Des",szRestrict.data());
	lpSrvIp			= new KRestrictedLine(this,"SIp","1234567890.");
	lpPortNum		= new KRestrictedLine(this,"Prt","1234567890");
	QLabel *lpL1	= new QLabel("Network:",this);
	QLabel *lpL2	= new QLabel("Server name:",this);
	QLabel *lpL3	= new QLabel("Server ip (optional):",this);
	QLabel *lpL4	= new QLabel("Port:",this);
	QLabel *lpL5	= new QLabel("Password (optional):",this);
	QLabel *lpL6	= new QLabel("Description:",this);
	lpOk			= new QPushButton("OK",this);
	lpCancel		= new QPushButton("Cancel",this);
	lpHelp          = new QPushButton("Help",this);

	lpNetName->setText(lpS->szNetworkName.data());
	lpSrvName->setText(lpS->szServerName.data());
	lpSrvIp->setText(lpS->szIp.data());
	QString szPort;
	szPort.setNum(lpS->iPort);
	lpPortNum->setText(szPort.data());
	lpPassword->setText(lpS->szPassword.data());
	lpDescription->setText(lpS->szDescription.data());

	lpSrvIp->setGeometry(160,70,150,26);
	lpSrvName->setGeometry(160,40,150,26);
	lpNetName->setGeometry(160,10,150,26);
	lpPortNum->setGeometry(160,100,150,26);
	lpPassword->setGeometry(160,130,150,26);
	lpDescription->setGeometry(160,160,150,26);
	lpL1->setGeometry(10,10,145,26);
	lpL2->setGeometry(10,40,145,26);
	lpL3->setGeometry(10,70,145,26);
	lpL4->setGeometry(10,100,145,26);
	lpL6->setGeometry(10,160,145,26);
	lpL5->setGeometry(10,130,145,26);
	lpOk->setGeometry(214,200,96,26);
	lpCancel->setGeometry(112,200,96,26);
	lpHelp->setGeometry(10,200,96,26);

	connect(lpOk,SIGNAL(clicked()),this,SLOT(applyChanges()));
	connect(lpCancel,SIGNAL(clicked()),this,SLOT(reject()));
	connect(lpHelp,SIGNAL(clicked()),this,SLOT(invokeHelp()));

	setFixedSize(320,240);
}
KviServerEditDialog::~KviServerEditDialog()
{
	delete lpNetName;
	delete lpSrvName;
	delete lpSrvIp;
	delete lpPortNum;
	delete lpPassword;
	delete lpDescription;
	delete lpOk;
	delete lpCancel;
	delete lpHelp;
}
void KviServerEditDialog::applyChanges()
{
	QString szServer=((lpNetName->text()) ? lpNetName->text() : "UnknownNet" );
	szServer=szServer.stripWhiteSpace();
	szServer+=':';
	QString szDummy=((lpSrvName->text()) ? lpSrvName->text() : "irc.unknown.net");
	szDummy=szDummy.stripWhiteSpace();
	szServer+=szDummy;
	szServer+=':';
	szDummy=((lpPortNum->text()) ? lpPortNum->text() : "6667");
	szDummy=szDummy.stripWhiteSpace();
	szServer+=szDummy;
	szServer+=':';
	szDummy=((lpSrvIp->text()) ? lpSrvIp->text() : "Resolve at connect");
	szDummy=szDummy.stripWhiteSpace();
	szServer+=szDummy;
	szServer+=':';
	szServer+=((lpPassword->text()) ? lpPassword->text() : "");
	szServer+=':';
	szServer+=((lpDescription->text()) ? lpDescription->text() : "Unknown location");
	m_lpS->fromString(szServer);
	accept();
}
void KviServerEditDialog::invokeHelp()
{
	_macro_kviApplication->invokeHTMLHelp("kvirc/seddlg.html",0);
}

//
//   Server settings dialog
//

KviSrvDialog::KviSrvDialog(KviFrame *lpParent,KviSrv *lpSrv):QTabDialog(lpParent,"KviServerDialog",true)
{
	m_lpSrv=lpSrv;
	//resize(500,410);
	setCaption("Servers...");

	lpSrvTab=new QWidget(this,"KviServerDialogTab1");

	lpSrvBox=new KTabListBox(lpSrvTab,0,6);
	lpSrvBox->setColumn(0,"Network",60);
	lpSrvBox->setColumn(1,"Server",150);
	lpSrvBox->setColumn(2,"Port",40);
	lpSrvBox->setColumn(3,"Ip",20);
	lpSrvBox->setColumn(4,"Password",20);
	lpSrvBox->setColumn(5,"Description",120);
	lpSrvBox->setSeparator(':');

	lpSrvBox->setGeometry(120,40,450,290);
	lpL1=new QLabel("Please note that KVIrc will start connecting from the top one...",lpSrvTab);
	lpL1->setGeometry(10,10,480,26);
	if(!m_lpSrv->serverList.isEmpty()){
		const char *szS=0;
		for(szS=m_lpSrv->serverList.first();szS;szS=m_lpSrv->serverList.next()){
			putBoxItem(szS);
		}
		lpSrvBox->setCurrentItem(0);
	} //else _debug_fatal("IsEmpty");
	lpMoveToTop	= new QPushButton("Move to top",lpSrvTab);
	lpMoveToTop->setGeometry(10,40,100,26);
	connect(lpMoveToTop,SIGNAL(clicked()),this,SLOT(moveToTop()));
	lpMoveUp	= new QPushButton("Move up",lpSrvTab);
	lpMoveUp->setGeometry(10,70,100,26);
	connect(lpMoveUp,SIGNAL(clicked()),this,SLOT(moveUp()));
	lpMoveDown	= new QPushButton("Move down",lpSrvTab);
	lpMoveDown->setGeometry(10,100,100,26);
	connect(lpMoveDown,SIGNAL(clicked()),this,SLOT(moveDown()));
	lpAdd		= new QPushButton("Add...",lpSrvTab);
	lpAdd->setGeometry(10,140,100,26);
	connect(lpAdd,SIGNAL(clicked()),this,SLOT(add()));
	lpRemove	= new QPushButton("Remove",lpSrvTab);
	lpRemove->setGeometry(10,170,100,26);
	connect(lpRemove,SIGNAL(clicked()),this,SLOT(remove()));
	lpEdit		= new QPushButton("Edit...",lpSrvTab);
	lpEdit->setGeometry(10,200,100,26);
	connect(lpEdit,SIGNAL(clicked()),this,SLOT(edit()));
	lpSave      = new QPushButton("Save",lpSrvTab);
	lpSave->setGeometry(10,240,100,26);
	connect(lpSave,SIGNAL(clicked()),this,SLOT(saveServers()));
	lpReadFromIni=new QPushButton("Read from *.ini",lpSrvTab);
	lpReadFromIni->setGeometry(10,270,100,26);
	connect(lpReadFromIni,SIGNAL(clicked()),this,SLOT(readFromIni()));
	lpSort=new QPushButton("Sort by network",lpSrvTab);
	lpSort->setGeometry(10,300,100,26);
	connect(lpSort,SIGNAL(clicked()),this,SLOT(sortServers()));

	lpMiscTab=new QWidget(this,"KviServerDialogTab2");
	lpL2=new QLabel("Connect timeout",lpMiscTab);
	lpL2->setGeometry(10,10,100,26);
	lpTimeout= new KIntegerLine(lpMiscTab);
	lpTimeout->setGeometry(120,10,450,26);
	lpTimeout->setValue(m_lpSrv->iTimeout);
	
	lpCache=new QCheckBox("Cache server lookups",lpMiscTab);
	lpCache->setGeometry(10,40,560,26);
	lpCache->setChecked(m_lpSrv->cacheLookup);

	lpROD=new QCheckBox("Reconnect on disconnect",lpMiscTab);
	lpROD->setGeometry(10,70,560,26);
	lpROD->setChecked(m_lpSrv->reconnectOnDisconnect);

	lpROCF=new QCheckBox("Reconnect on connect failed",lpMiscTab);
	lpROCF->setGeometry(10,100,560,26);
	lpROCF->setChecked(m_lpSrv->reconnectOnConnectFailed);

	KSeparator *lpSep1=new KSeparator(KSeparator::HLine,lpMiscTab);
	lpSep1->setGeometry(10,130,560,4);

	lpUseProxy      = new KviBoolChanger(this,lpMiscTab,10,140,560,26,"Use proxy server",&(m_lpSrv->bUseProxy));
	lpProxyHost     = new QLineEdit(lpMiscTab);
	lpProxyIp       = new QLabel(m_lpSrv->szProxyIp.data(),lpMiscTab);
	lpProxyUsername = new QLineEdit(lpMiscTab);
	lpProxyPassword = new QLineEdit(lpMiscTab);
	lpProxyPort     = new KIntegerLine(lpMiscTab);

	QLabel *lpL1    = new QLabel("Proxy host :",lpMiscTab);
	QLabel *lpL2    = new QLabel("Last used  :",lpMiscTab);
	QLabel *lpL3    = new QLabel("Port       :",lpMiscTab);
	QLabel *lpL4    = new QLabel("Username   :",lpMiscTab);
	QLabel *lpL5    = new QLabel("Password   :",lpMiscTab);

	lpL1->setGeometry(10,170,95,26);
	lpProxyHost->setGeometry(120,170,450,26);
	lpL2->setGeometry(10,200,95,26);
	lpProxyIp->setGeometry(120,200,450,26);
	lpL3->setGeometry(10,230,95,26);
	lpProxyPort->setGeometry(120,230,450,26);
	lpL4->setGeometry(10,260,95,26);
	lpProxyUsername->setGeometry(120,260,450,26);
	lpL5->setGeometry(10,290,95,26);
	lpProxyPassword->setGeometry(120,290,450,26);

	lpProxyIp->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	lpProxyHost->setText(m_lpSrv->szProxyAddress.data());
	bool bOk=false;
	unsigned short int val=m_lpSrv->szProxyPort.toUShort(&bOk);
	if(!bOk)val=1080;
	lpProxyPort->setValue(val);
	lpProxyUsername->setText(m_lpSrv->szProxyUsername.data());
	lpProxyPassword->setText(m_lpSrv->szProxyPassword.data());

	connect(lpUseProxy,SIGNAL(toggled(bool)),lpProxyHost,SLOT(setEnabled(bool)));
	connect(lpUseProxy,SIGNAL(toggled(bool)),lpProxyIp,SLOT(setEnabled(bool)));
	connect(lpUseProxy,SIGNAL(toggled(bool)),lpProxyPort,SLOT(setEnabled(bool)));
	connect(lpUseProxy,SIGNAL(toggled(bool)),lpProxyUsername,SLOT(setEnabled(bool)));
	connect(lpUseProxy,SIGNAL(toggled(bool)),lpProxyPassword,SLOT(setEnabled(bool)));

	lpProxyHost->setEnabled(m_lpSrv->bUseProxy);
	lpProxyIp->setEnabled(m_lpSrv->bUseProxy);
	lpProxyPort->setEnabled(m_lpSrv->bUseProxy);
	lpProxyUsername->setEnabled(m_lpSrv->bUseProxy);
	lpProxyPassword->setEnabled(m_lpSrv->bUseProxy);

	addTab(lpSrvTab,"Server list");
	addTab(lpMiscTab,"Proxy and misc options");

	setDefaultButton("Help");
	setCancelButton();
	connect(this,SIGNAL(cancelButtonPressed()),this,SLOT(reject()));
	connect(this,SIGNAL(defaultButtonPressed()),this,SLOT(invokeHelp()));
	connect(this,SIGNAL(applyButtonPressed()),this,SLOT(applySlot()));

	QResizeEvent e(size(),size());
	resizeEvent(&e);
	
}

KviSrvDialog::~KviSrvDialog()
{
	delete lpUseProxy;
	delete lpProxyHost;
	delete lpProxyIp;
	delete lpProxyUsername;
	delete lpProxyPassword;
	delete lpProxyPort;
	delete lpMoveToTop;
	delete lpMoveUp;
	delete lpMoveDown;
	delete lpAdd;
	delete lpRemove;
	delete lpEdit;
	delete lpL1;
	delete lpSave;
	delete lpSrvBox;
	delete lpSort;
	delete lpReadFromIni;
	delete lpSrvTab;
	delete lpTimeout;
	delete lpROD;
	delete lpROCF;
	delete lpCache;
	delete lpL2;
	delete lpMiscTab;
}
//============ resizeEvent ============//
void KviSrvDialog::resizeEvent(QResizeEvent *)
{
	setFixedSize(600,410);
}


//============ invokeHelp ============//
void KviSrvDialog::invokeHelp()
{
	_macro_kviApplication->invokeHTMLHelp("kvirc/srvdlg.html",0);
}
//============ sortServers ============//
void KviSrvDialog::sortServers()
{
	_debug_entertrace("sortServers");
	if(KMsgBox::yesNo(0,"Sort servers.","This implies accepting the changes made\nProceed anyway?")!=1)return;
	applySlot();
	QStrList aList;
	aList=m_lpSrv->serverList; //deep copy
	while(!m_lpSrv->serverList.isEmpty())m_lpSrv->serverList.removeLast();
	const char *szS=0;
	
	for(szS=aList.first();szS;szS=aList.next()){
		m_lpSrv->serverList.inSort(szS);
	}


	lpSrvBox->clear();
	lpSrvBox->setAutoUpdate(false);
	if(!m_lpSrv->serverList.isEmpty()){
		for(szS=m_lpSrv->serverList.first();szS;szS=m_lpSrv->serverList.next()){
			putBoxItem(szS);
		}
		lpSrvBox->setCurrentItem(0);
	}
	lpSrvBox->setAutoUpdate(true);
	lpSrvBox->repaint();
	_debug_leavetrace("sortServers");
}

//============ readFromIni ============//
void KviSrvDialog::readFromIni()
{
	_debug_entertrace("readFromIni");
	if(KMsgBox::yesNo(0,"Read from mIrc's *.ini file.","This operation can NOT be undone.\nProceed anyway?")==1){
		applySlot();
		QString szFile=KFileDialog::getOpenFileName();
		if(!szFile.isEmpty()){
			m_lpSrv->readFromMircServersIni(szFile.data());
		}
		lpSrvBox->clear();
		lpSrvBox->setAutoUpdate(false);
		if(!m_lpSrv->serverList.isEmpty()){
			const char *szS=0;
			for(szS=m_lpSrv->serverList.first();szS;szS=m_lpSrv->serverList.next()){
				putBoxItem(szS);
			}
			lpSrvBox->setCurrentItem(0);
		}
		lpSrvBox->setAutoUpdate(true);
		lpSrvBox->repaint();
	}
	_debug_leavetrace("readFromIni");
}
//============ saveServers ============//
void KviSrvDialog::saveServers()
{
	_debug_entertrace("saveServers");
	m_lpSrv->saveSettings();
	_debug_leavetrace("saveServers");
}

//============ putBoxItem ============//

void KviSrvDialog::putBoxItem(const char *szItem,int index)
{
	_debug_entertrace("putBoxItem");
	QString szI=szItem;
	lpSrvBox->insertItem("a:s:d:f:g:h",index);
	//network
	if(index==-1)index=(lpSrvBox->count() -1);
	KviServer srv(szI);
	lpSrvBox->changeItemPart(srv.szNetworkName.data(),index,0);
	lpSrvBox->changeItemPart(srv.szServerName.data(),index,1);
	QString szPort;
	szPort.setNum(srv.iPort);
	lpSrvBox->changeItemPart(szPort.data(),index,2);
	lpSrvBox->changeItemPart(srv.szIp.data(),index,3);
	lpSrvBox->changeItemPart(srv.szPassword.data(),index,4);
	lpSrvBox->changeItemPart(srv.szDescription.data(),index,5);
	_debug_leavetrace("putBoxItem");
}

void KviSrvDialog::moveToTop(){
	int nCur=lpSrvBox->currentItem();
	if(nCur != -1){
		char szS[1024];
		const char *itm=lpSrvBox->text(nCur);
		if(itm){
			strcpy(szS,itm);
			lpSrvBox->removeItem(nCur);
			putBoxItem(szS,0);
			lpSrvBox->setCurrentItem(nCur);
			lpSrvBox->update();
		}
	}
}
void KviSrvDialog::moveUp(){
	int nCur=lpSrvBox->currentItem();
	if(nCur > 0){
		char szS[1024];
		const char *itm=lpSrvBox->text(nCur);
		if(itm){
			strcpy(szS,itm);
			lpSrvBox->removeItem(nCur);
			putBoxItem(szS,nCur-1);
			lpSrvBox->setCurrentItem(nCur-1);
			lpSrvBox->update();
		}
	}
}
void KviSrvDialog::moveDown(){
	int nCur=lpSrvBox->currentItem();
	if(nCur >= 0 && nCur<(int)(lpSrvBox->count()-1)){
		char szS[1024];
		const char *itm=lpSrvBox->text(nCur);
		if(itm){
			strcpy(szS,itm);
			lpSrvBox->removeItem(nCur);
			putBoxItem(szS,nCur+1);
			lpSrvBox->setCurrentItem(nCur+1);
			lpSrvBox->update();
		}
	}
}
void KviSrvDialog::edit(){
	int nCur=lpSrvBox->currentItem();
	if(nCur>=0){
		char szS[1024];
		const char *itm=lpSrvBox->text(nCur);
		if(itm){
			strcpy(szS,itm);
			lpSrvBox->removeItem(nCur);
			QString szServer(szS);
			KviServer srv(szServer);
			KviServerEditDialog theDlg(this,&srv);
			theDlg.exec();
			srv.toString(szServer);
			putBoxItem(szServer.data(),nCur);
			lpSrvBox->setCurrentItem(nCur);
			lpSrvBox->update();
		}
	}
}
void KviSrvDialog::remove(){
	int nCur=lpSrvBox->currentItem();
	if(nCur>=0){
		lpSrvBox->removeItem(nCur);
		if((int)lpSrvBox->count()>nCur)lpSrvBox->setCurrentItem(nCur);
		lpSrvBox->update();
	}
}
void KviSrvDialog::add(){
	int nCur=lpSrvBox->currentItem();
	putBoxItem("UnknownNet:irc.unknown.net:6667:Resolve at connect::UnknownLocation",nCur+1);
	lpSrvBox->setCurrentItem(nCur+1);
	lpSrvBox->update();
	edit();
}

void KviSrvDialog::applySlot(){
	while(!m_lpSrv->serverList.isEmpty())m_lpSrv->serverList.removeLast();
	uint cnt=lpSrvBox->count();
	for(uint i=0;i<cnt;i++){
		m_lpSrv->serverList.append(lpSrvBox->text(i));
	}
	if((lpTimeout->value()>5) && (lpTimeout->value()<300))m_lpSrv->iTimeout=lpTimeout->value();
	else m_lpSrv->iTimeout=30;
	m_lpSrv->cacheLookup=lpCache->isChecked();
	m_lpSrv->reconnectOnDisconnect=lpROD->isChecked();
	m_lpSrv->reconnectOnConnectFailed=lpROCF->isChecked();
	QString szAddr=( lpProxyHost->text() ? lpProxyHost->text() : "");
	if(szAddr != m_lpSrv->szProxyAddress)m_lpSrv->szProxyIp="";
	if(szAddr.isEmpty())m_lpSrv->bUseProxy=false;
	m_lpSrv->szProxyAddress=szAddr.copy();
	unsigned short int val=lpProxyPort->value();
	if(val==0)val=1080;
	m_lpSrv->szProxyPort.setNum(val);
	m_lpSrv->szProxyUsername=(lpProxyUsername->text() ? lpProxyUsername->text() : "");
	m_lpSrv->szProxyPassword=(lpProxyPassword->text() ? lpProxyPassword->text() : "");
}

#include "m_kvi_srvdlg.moc"
